@@ -3,16 +3,16 @@ package continuous_querier
 import (
 	"errors"
 	"fmt"
-	"io/ioutil"
-	"log"
+	"os"
 	"sync"
 	"testing"
 	"time"
 
-	"github.com/influxdata/influxdb/cluster"
-	"github.com/influxdata/influxdb/influxql"
+	"github.com/influxdata/influxdb/logger"
 	"github.com/influxdata/influxdb/models"
+	"github.com/influxdata/influxdb/query"
 	"github.com/influxdata/influxdb/services/meta"
+	"github.com/influxdata/influxql"
 )
 
 var (
@@ -50,12 +50,12 @@ func TestContinuousQueryService_Run(t *testing.T) {
 
 	// Set a callback for ExecuteStatement.
 	s.QueryExecutor.StatementExecutor = &StatementExecutor{
-		ExecuteStatementFn: func(stmt influxql.Statement, ctx *influxql.ExecutionContext) error {
+		ExecuteStatementFn: func(stmt influxql.Statement, ctx query.ExecutionContext) error {
 			callCnt++
 			if callCnt >= expectCallCnt {
 				done <- struct{}{}
 			}
-			ctx.Results <- &influxql.Result{}
+			ctx.Results <- &query.Result{}
 			return nil
 		},
 	}
@@ -115,17 +115,24 @@ func TestContinuousQueryService_ResampleOptions(t *testing.T) {
 	s.RunInterval = 10 * time.Minute
 
 	done := make(chan struct{})
-	expectCallCnt := 0
-	callCnt := 0
+	var expected struct {
+		min time.Time
+		max time.Time
+	}
 
 	// Set a callback for ExecuteStatement.
 	s.QueryExecutor.StatementExecutor = &StatementExecutor{
-		ExecuteStatementFn: func(stmt influxql.Statement, ctx *influxql.ExecutionContext) error {
-			callCnt++
-			if callCnt >= expectCallCnt {
-				done <- struct{}{}
+		ExecuteStatementFn: func(stmt influxql.Statement, ctx query.ExecutionContext) error {
+			s := stmt.(*influxql.SelectStatement)
+			valuer := &influxql.NowValuer{Location: s.Location}
+			_, timeRange, err := influxql.ConditionExpr(s.Condition, valuer)
+			if err != nil {
+				t.Errorf("unexpected error parsing time range: %s", err)
+			} else if !expected.min.Equal(timeRange.Min) || !expected.max.Equal(timeRange.Max) {
+				t.Errorf("mismatched time range: got=(%s, %s) exp=(%s, %s)", timeRange.Min, timeRange.Max, expected.min, expected.max)
 			}
-			ctx.Results <- &influxql.Result{}
+			done <- struct{}{}
+			ctx.Results <- &query.Result{}
 			return nil
 		},
 	}
@@ -135,8 +142,9 @@ func TestContinuousQueryService_ResampleOptions(t *testing.T) {
 
 	// Set the 'now' time to the start of a 10 minute interval. Then trigger a run.
 	// This should trigger two queries (one for the current time interval, one for the previous).
-	now := time.Now().Truncate(10 * time.Minute)
-	expectCallCnt += 2
+	now := time.Now().UTC().Truncate(10 * time.Minute)
+	expected.min = now.Add(-2 * time.Minute)
+	expected.max = now.Add(-1)
 	s.RunCh <- &RunRequest{Now: now}
 
 	if err := wait(done, 100*time.Millisecond); err != nil {
@@ -145,7 +153,8 @@ func TestContinuousQueryService_ResampleOptions(t *testing.T) {
 
 	// Trigger another run 10 seconds later. Another two queries should happen,
 	// but it will be a different two queries.
-	expectCallCnt += 2
+	expected.min = expected.min.Add(time.Minute)
+	expected.max = expected.max.Add(time.Minute)
 	s.RunCh <- &RunRequest{Now: now.Add(10 * time.Second)}
 
 	if err := wait(done, 100*time.Millisecond); err != nil {
@@ -155,13 +164,22 @@ func TestContinuousQueryService_ResampleOptions(t *testing.T) {
 	// Reset the time period and send the initial request at 5 seconds after the
 	// 10 minute mark. There should be exactly one call since the current interval is too
 	// young and only one interval matches the FOR duration.
-	expectCallCnt += 1
+	expected.min = now.Add(-time.Minute)
+	expected.max = now.Add(-1)
 	s.Run("", "", now.Add(5*time.Second))
 
 	if err := wait(done, 100*time.Millisecond); err != nil {
 		t.Fatal(err)
 	}
 
+	// Send a message 10 minutes later and ensure that the system plays catchup.
+	expected.max = now.Add(10*time.Minute - 1)
+	s.RunCh <- &RunRequest{Now: now.Add(10 * time.Minute)}
+
+	if err := wait(done, 100*time.Millisecond); err != nil {
+		t.Fatal(err)
+	}
+
 	// No overflow should be sent.
 	if err := wait(done, 100*time.Millisecond); err == nil {
 		t.Error("too many queries executed")
@@ -179,17 +197,24 @@ func TestContinuousQueryService_EveryHigherThanInterval(t *testing.T) {
 	s.RunInterval = 10 * time.Minute
 
 	done := make(chan struct{})
-	expectCallCnt := 0
-	callCnt := 0
+	var expected struct {
+		min time.Time
+		max time.Time
+	}
 
 	// Set a callback for ExecuteQuery.
 	s.QueryExecutor.StatementExecutor = &StatementExecutor{
-		ExecuteStatementFn: func(stmt influxql.Statement, ctx *influxql.ExecutionContext) error {
-			callCnt++
-			if callCnt >= expectCallCnt {
-				done <- struct{}{}
+		ExecuteStatementFn: func(stmt influxql.Statement, ctx query.ExecutionContext) error {
+			s := stmt.(*influxql.SelectStatement)
+			valuer := &influxql.NowValuer{Location: s.Location}
+			_, timeRange, err := influxql.ConditionExpr(s.Condition, valuer)
+			if err != nil {
+				t.Errorf("unexpected error parsing time range: %s", err)
+			} else if !expected.min.Equal(timeRange.Min) || !expected.max.Equal(timeRange.Max) {
+				t.Errorf("mismatched time range: got=(%s, %s) exp=(%s, %s)", timeRange.Min, timeRange.Max, expected.min, expected.max)
 			}
-			ctx.Results <- &influxql.Result{}
+			done <- struct{}{}
+			ctx.Results <- &query.Result{}
 			return nil
 		},
 	}
@@ -201,7 +226,8 @@ func TestContinuousQueryService_EveryHigherThanInterval(t *testing.T) {
 	// This should trigger two queries (one for the current time interval, one for the previous)
 	// since the default FOR interval should be EVERY, not the GROUP BY interval.
 	now := time.Now().Truncate(10 * time.Minute)
-	expectCallCnt += 2
+	expected.min = now.Add(-time.Minute)
+	expected.max = now.Add(-1)
 	s.RunCh <- &RunRequest{Now: now}
 
 	if err := wait(done, 100*time.Millisecond); err != nil {
@@ -216,7 +242,8 @@ func TestContinuousQueryService_EveryHigherThanInterval(t *testing.T) {
 	}
 
 	// Run again 1 minute later. Another two queries should run.
-	expectCallCnt += 2
+	expected.min = now
+	expected.max = now.Add(time.Minute - 1)
 	s.RunCh <- &RunRequest{Now: now.Add(time.Minute)}
 
 	if err := wait(done, 100*time.Millisecond); err != nil {
@@ -229,6 +256,55 @@ func TestContinuousQueryService_EveryHigherThanInterval(t *testing.T) {
 	}
 }
 
+func TestContinuousQueryService_GroupByOffset(t *testing.T) {
+	s := NewTestService(t)
+	mc := NewMetaClient(t)
+	mc.CreateDatabase("db", "")
+	mc.CreateContinuousQuery("db", "cq", `CREATE CONTINUOUS QUERY cq ON db BEGIN SELECT mean(value) INTO cpu_mean FROM cpu GROUP BY time(1m, 30s) END`)
+	s.MetaClient = mc
+
+	// Set RunInterval high so we can trigger using Run method.
+	s.RunInterval = 10 * time.Minute
+
+	done := make(chan struct{})
+	var expected struct {
+		min time.Time
+		max time.Time
+	}
+
+	// Set a callback for ExecuteStatement.
+	s.QueryExecutor.StatementExecutor = &StatementExecutor{
+		ExecuteStatementFn: func(stmt influxql.Statement, ctx query.ExecutionContext) error {
+			s := stmt.(*influxql.SelectStatement)
+			valuer := &influxql.NowValuer{Location: s.Location}
+			_, timeRange, err := influxql.ConditionExpr(s.Condition, valuer)
+			if err != nil {
+				t.Errorf("unexpected error parsing time range: %s", err)
+			} else if !expected.min.Equal(timeRange.Min) || !expected.max.Equal(timeRange.Max) {
+				t.Errorf("mismatched time range: got=(%s, %s) exp=(%s, %s)", timeRange.Min, timeRange.Max, expected.min, expected.max)
+			}
+			done <- struct{}{}
+			ctx.Results <- &query.Result{}
+			return nil
+		},
+	}
+
+	s.Open()
+	defer s.Close()
+
+	// Set the 'now' time to the start of a 10 minute interval with a 30 second offset.
+	// Then trigger a run. This should trigger two queries (one for the current time
+	// interval, one for the previous).
+	now := time.Now().UTC().Truncate(10 * time.Minute).Add(30 * time.Second)
+	expected.min = now.Add(-time.Minute)
+	expected.max = now.Add(-1)
+	s.RunCh <- &RunRequest{Now: now}
+
+	if err := wait(done, 100*time.Millisecond); err != nil {
+		t.Fatal(err)
+	}
+}
+
 // Test service when not the cluster leader (CQs shouldn't run).
 func TestContinuousQueryService_NotLeader(t *testing.T) {
 	s := NewTestService(t)
@@ -239,36 +315,9 @@ func TestContinuousQueryService_NotLeader(t *testing.T) {
 	done := make(chan struct{})
 	// Set a callback for ExecuteStatement. Shouldn't get called because we're not the leader.
 	s.QueryExecutor.StatementExecutor = &StatementExecutor{
-		ExecuteStatementFn: func(stmt influxql.Statement, ctx *influxql.ExecutionContext) error {
+		ExecuteStatementFn: func(stmt influxql.Statement, ctx query.ExecutionContext) error {
 			done <- struct{}{}
-			ctx.Results <- &influxql.Result{Err: errUnexpected}
-			return nil
-		},
-	}
-
-	s.Open()
-	// Trigger service to run CQs.
-	s.RunCh <- &RunRequest{Now: time.Now()}
-	// Expect timeout error because ExecuteQuery callback wasn't called.
-	if err := wait(done, 100*time.Millisecond); err == nil {
-		t.Error(err)
-	}
-	s.Close()
-}
-
-// Test service behavior when meta store fails to get databases.
-func TestContinuousQueryService_MetaClientFailsToGetDatabases(t *testing.T) {
-	s := NewTestService(t)
-	// Set RunInterval high so we can test triggering with the RunCh below.
-	s.RunInterval = 10 * time.Second
-	s.MetaClient.(*MetaClient).Err = errExpected
-
-	done := make(chan struct{})
-	// Set ExecuteQuery callback, which shouldn't get called because of meta store failure.
-	s.QueryExecutor.StatementExecutor = &StatementExecutor{
-		ExecuteStatementFn: func(stmt influxql.Statement, ctx *influxql.ExecutionContext) error {
-			done <- struct{}{}
-			ctx.Results <- &influxql.Result{Err: errUnexpected}
+			ctx.Results <- &query.Result{Err: errUnexpected}
 			return nil
 		},
 	}
@@ -287,7 +336,7 @@ func TestContinuousQueryService_MetaClientFailsToGetDatabases(t *testing.T) {
 func TestExecuteContinuousQuery_InvalidQueries(t *testing.T) {
 	s := NewTestService(t)
 	s.QueryExecutor.StatementExecutor = &StatementExecutor{
-		ExecuteStatementFn: func(stmt influxql.Statement, ctx *influxql.ExecutionContext) error {
+		ExecuteStatementFn: func(stmt influxql.Statement, ctx query.ExecutionContext) error {
 			return errUnexpected
 		},
 	}
@@ -296,31 +345,254 @@ func TestExecuteContinuousQuery_InvalidQueries(t *testing.T) {
 	cqi := dbi.ContinuousQueries[0]
 
 	cqi.Query = `this is not a query`
-	err := s.ExecuteContinuousQuery(&dbi, &cqi, time.Now())
-	if err == nil {
+	if _, err := s.ExecuteContinuousQuery(&dbi, &cqi, time.Now()); err == nil {
 		t.Error("expected error but got nil")
 	}
 
 	// Valid query but invalid continuous query.
 	cqi.Query = `SELECT * FROM cpu`
-	err = s.ExecuteContinuousQuery(&dbi, &cqi, time.Now())
-	if err == nil {
+	if _, err := s.ExecuteContinuousQuery(&dbi, &cqi, time.Now()); err == nil {
 		t.Error("expected error but got nil")
 	}
 
 	// Group by requires aggregate.
 	cqi.Query = `SELECT value INTO other_value FROM cpu WHERE time > now() - 1h GROUP BY time(1s)`
-	err = s.ExecuteContinuousQuery(&dbi, &cqi, time.Now())
-	if err == nil {
+	if _, err := s.ExecuteContinuousQuery(&dbi, &cqi, time.Now()); err == nil {
 		t.Error("expected error but got nil")
 	}
 }
 
+// Test the time range for different CQ durations.
+func TestExecuteContinuousQuery_TimeRange(t *testing.T) {
+	// Choose a start date that is not on an interval border for anyone.
+	now := mustParseTime(t, "2000-01-01T00:00:00Z")
+	for _, tt := range []struct {
+		d          string
+		start, end time.Time
+	}{
+		{
+			d:     "10s",
+			start: mustParseTime(t, "2000-01-01T00:00:00Z"),
+			end:   mustParseTime(t, "2000-01-01T00:00:10Z"),
+		},
+		{
+			d:     "1m",
+			start: mustParseTime(t, "2000-01-01T00:00:00Z"),
+			end:   mustParseTime(t, "2000-01-01T00:01:00Z"),
+		},
+		{
+			d:     "10m",
+			start: mustParseTime(t, "2000-01-01T00:00:00Z"),
+			end:   mustParseTime(t, "2000-01-01T00:10:00Z"),
+		},
+		{
+			d:     "30m",
+			start: mustParseTime(t, "2000-01-01T00:00:00Z"),
+			end:   mustParseTime(t, "2000-01-01T00:30:00Z"),
+		},
+		{
+			d:     "1h",
+			start: mustParseTime(t, "2000-01-01T00:00:00Z"),
+			end:   mustParseTime(t, "2000-01-01T01:00:00Z"),
+		},
+		{
+			d:     "2h",
+			start: mustParseTime(t, "2000-01-01T00:00:00Z"),
+			end:   mustParseTime(t, "2000-01-01T02:00:00Z"),
+		},
+		{
+			d:     "12h",
+			start: mustParseTime(t, "2000-01-01T00:00:00Z"),
+			end:   mustParseTime(t, "2000-01-01T12:00:00Z"),
+		},
+		{
+			d:     "1d",
+			start: mustParseTime(t, "2000-01-01T00:00:00Z"),
+			end:   mustParseTime(t, "2000-01-02T00:00:00Z"),
+		},
+		{
+			d:     "1w",
+			start: mustParseTime(t, "1999-12-30T00:00:00Z"),
+			end:   mustParseTime(t, "2000-01-06T00:00:00Z"),
+		},
+	} {
+		t.Run(tt.d, func(t *testing.T) {
+			d, err := influxql.ParseDuration(tt.d)
+			if err != nil {
+				t.Fatalf("unable to parse duration: %s", err)
+			}
+
+			s := NewTestService(t)
+			mc := NewMetaClient(t)
+			mc.CreateDatabase("db", "")
+			mc.CreateContinuousQuery("db", "cq",
+				fmt.Sprintf(`CREATE CONTINUOUS QUERY cq ON db BEGIN SELECT mean(value) INTO cpu_mean FROM cpu GROUP BY time(%s) END`, tt.d))
+			s.MetaClient = mc
+
+			// Set RunInterval high so we can trigger using Run method.
+			s.RunInterval = 10 * time.Minute
+			done := make(chan struct{})
+
+			// Set a callback for ExecuteStatement.
+			s.QueryExecutor.StatementExecutor = &StatementExecutor{
+				ExecuteStatementFn: func(stmt influxql.Statement, ctx query.ExecutionContext) error {
+					s := stmt.(*influxql.SelectStatement)
+					valuer := &influxql.NowValuer{Location: s.Location}
+					_, timeRange, err := influxql.ConditionExpr(s.Condition, valuer)
+					timeRange.Max = timeRange.Max.Add(time.Nanosecond)
+					if err != nil {
+						t.Errorf("unexpected error parsing time range: %s", err)
+					} else if !tt.start.Equal(timeRange.Min) || !tt.end.Equal(timeRange.Max) {
+						t.Errorf("mismatched time range: got=(%s, %s) exp=(%s, %s)", timeRange.Min, timeRange.Max, tt.start, tt.end)
+					}
+					done <- struct{}{}
+					ctx.Results <- &query.Result{}
+					return nil
+				},
+			}
+
+			s.Open()
+			defer s.Close()
+
+			// Send an initial run request one nanosecond after the start to
+			// prime the last CQ map.
+			s.RunCh <- &RunRequest{Now: now.Add(time.Nanosecond)}
+			// Execute the real request after the time interval.
+			s.RunCh <- &RunRequest{Now: now.Add(d)}
+			if err := wait(done, 100*time.Millisecond); err != nil {
+				t.Fatal(err)
+			}
+		})
+	}
+}
+
+// Test the time range for different CQ durations.
+func TestExecuteContinuousQuery_TimeZone(t *testing.T) {
+	type test struct {
+		now        time.Time
+		start, end time.Time
+	}
+
+	// Choose a start date that is not on an interval border for anyone.
+	for _, tt := range []struct {
+		name    string
+		d       string
+		options string
+		initial time.Time
+		tests   []test
+	}{
+		{
+			name:    "DaylightSavingsStart/1d",
+			d:       "1d",
+			initial: mustParseTime(t, "2000-04-02T00:00:00-05:00"),
+			tests: []test{
+				{
+					start: mustParseTime(t, "2000-04-02T00:00:00-05:00"),
+					end:   mustParseTime(t, "2000-04-03T00:00:00-04:00"),
+				},
+			},
+		},
+		{
+			name:    "DaylightSavingsStart/2h",
+			d:       "2h",
+			initial: mustParseTime(t, "2000-04-02T00:00:00-05:00"),
+			tests: []test{
+				{
+					start: mustParseTime(t, "2000-04-02T00:00:00-05:00"),
+					end:   mustParseTime(t, "2000-04-02T03:00:00-04:00"),
+				},
+				{
+					start: mustParseTime(t, "2000-04-02T03:00:00-04:00"),
+					end:   mustParseTime(t, "2000-04-02T04:00:00-04:00"),
+				},
+			},
+		},
+		{
+			name:    "DaylightSavingsEnd/1d",
+			d:       "1d",
+			initial: mustParseTime(t, "2000-10-29T00:00:00-04:00"),
+			tests: []test{
+				{
+					start: mustParseTime(t, "2000-10-29T00:00:00-04:00"),
+					end:   mustParseTime(t, "2000-10-30T00:00:00-05:00"),
+				},
+			},
+		},
+		{
+			name:    "DaylightSavingsEnd/2h",
+			d:       "2h",
+			initial: mustParseTime(t, "2000-10-29T00:00:00-04:00"),
+			tests: []test{
+				{
+					start: mustParseTime(t, "2000-10-29T00:00:00-04:00"),
+					end:   mustParseTime(t, "2000-10-29T02:00:00-05:00"),
+				},
+				{
+					start: mustParseTime(t, "2000-10-29T02:00:00-05:00"),
+					end:   mustParseTime(t, "2000-10-29T04:00:00-05:00"),
+				},
+			},
+		},
+	} {
+		t.Run(tt.name, func(t *testing.T) {
+			s := NewTestService(t)
+			mc := NewMetaClient(t)
+			mc.CreateDatabase("db", "")
+			mc.CreateContinuousQuery("db", "cq",
+				fmt.Sprintf(`CREATE CONTINUOUS QUERY cq ON db %s BEGIN SELECT mean(value) INTO cpu_mean FROM cpu GROUP BY time(%s) TZ('America/New_York') END`, tt.options, tt.d))
+			s.MetaClient = mc
+
+			// Set RunInterval high so we can trigger using Run method.
+			s.RunInterval = 10 * time.Minute
+			done := make(chan struct{})
+
+			// Set a callback for ExecuteStatement.
+			tests := make(chan test, 1)
+			s.QueryExecutor.StatementExecutor = &StatementExecutor{
+				ExecuteStatementFn: func(stmt influxql.Statement, ctx query.ExecutionContext) error {
+					test := <-tests
+					s := stmt.(*influxql.SelectStatement)
+					valuer := &influxql.NowValuer{Location: s.Location}
+					_, timeRange, err := influxql.ConditionExpr(s.Condition, valuer)
+					timeRange.Max = timeRange.Max.Add(time.Nanosecond)
+					if err != nil {
+						t.Errorf("unexpected error parsing time range: %s", err)
+					} else if !test.start.Equal(timeRange.Min) || !test.end.Equal(timeRange.Max) {
+						t.Errorf("mismatched time range: got=(%s, %s) exp=(%s, %s)", timeRange.Min, timeRange.Max, test.start, test.end)
+					}
+					done <- struct{}{}
+					ctx.Results <- &query.Result{}
+					return nil
+				},
+			}
+
+			s.Open()
+			defer s.Close()
+
+			// Send an initial run request one nanosecond after the start to
+			// prime the last CQ map.
+			s.RunCh <- &RunRequest{Now: tt.initial.Add(time.Nanosecond)}
+			// Execute each of the tests and ensure the times are correct.
+			for i, test := range tt.tests {
+				tests <- test
+				now := test.now
+				if now.IsZero() {
+					now = test.end
+				}
+				s.RunCh <- &RunRequest{Now: now}
+				if err := wait(done, 100*time.Millisecond); err != nil {
+					t.Fatal(fmt.Errorf("%d. %s", i+1, err))
+				}
+			}
+		})
+	}
+}
+
 // Test ExecuteContinuousQuery when QueryExecutor returns an error.
 func TestExecuteContinuousQuery_QueryExecutor_Error(t *testing.T) {
 	s := NewTestService(t)
 	s.QueryExecutor.StatementExecutor = &StatementExecutor{
-		ExecuteStatementFn: func(stmt influxql.Statement, ctx *influxql.ExecutionContext) error {
+		ExecuteStatementFn: func(stmt influxql.Statement, ctx query.ExecutionContext) error {
 			return errExpected
 		},
 	}
@@ -330,23 +602,96 @@ func TestExecuteContinuousQuery_QueryExecutor_Error(t *testing.T) {
 	cqi := dbi.ContinuousQueries[0]
 
 	now := time.Now().Truncate(10 * time.Minute)
-	err := s.ExecuteContinuousQuery(&dbi, &cqi, now)
-	if err != errExpected {
+	if _, err := s.ExecuteContinuousQuery(&dbi, &cqi, now); err != errExpected {
 		t.Errorf("exp = %s, got = %v", errExpected, err)
 	}
 }
 
+func TestService_ExecuteContinuousQuery_LogsToMonitor(t *testing.T) {
+	s := NewTestService(t)
+	const writeN = int64(50)
+
+	s.QueryExecutor.StatementExecutor = &StatementExecutor{
+		ExecuteStatementFn: func(stmt influxql.Statement, ctx query.ExecutionContext) error {
+			ctx.Results <- &query.Result{
+				Series: []*models.Row{{
+					Name:    "result",
+					Columns: []string{"time", "written"},
+					Values:  [][]interface{}{{time.Time{}, writeN}},
+				}},
+			}
+			return nil
+		},
+	}
+	s.queryStatsEnabled = true
+	var point models.Point
+	s.Monitor = &monitor{
+		EnabledFn: func() bool { return true },
+		WritePointsFn: func(p models.Points) error {
+			if len(p) != 1 {
+				t.Fatalf("expected point")
+			}
+			point = p[0]
+			return nil
+		},
+	}
+
+	dbis := s.MetaClient.Databases()
+	dbi := dbis[0]
+	cqi := dbi.ContinuousQueries[0]
+
+	now := time.Now().Truncate(10 * time.Minute)
+	if ok, err := s.ExecuteContinuousQuery(&dbi, &cqi, now); !ok || err != nil {
+		t.Fatalf("ExecuteContinuousQuery failed, ok=%t, err=%v", ok, err)
+	}
+
+	if point == nil {
+		t.Fatal("expected Monitor.WritePoints call")
+	}
+
+	f, _ := point.Fields()
+	if got, ok := f["pointsWrittenOK"].(int64); !ok || got != writeN {
+		t.Errorf("unexpected value for written; exp=%d, got=%d", writeN, got)
+	}
+}
+
+func TestService_ExecuteContinuousQuery_LogToMonitor_DisabledByDefault(t *testing.T) {
+	s := NewTestService(t)
+	s.QueryExecutor.StatementExecutor = &StatementExecutor{
+		ExecuteStatementFn: func(stmt influxql.Statement, ctx query.ExecutionContext) error {
+			ctx.Results <- &query.Result{}
+			return nil
+		},
+	}
+	s.Monitor = &monitor{
+		EnabledFn: func() bool { return true },
+		WritePointsFn: func(p models.Points) error {
+			t.Fatalf("unexpected Monitor.WritePoints call")
+			return nil
+		},
+	}
+
+	dbis := s.MetaClient.Databases()
+	dbi := dbis[0]
+	cqi := dbi.ContinuousQueries[0]
+
+	now := time.Now().Truncate(10 * time.Minute)
+	if ok, err := s.ExecuteContinuousQuery(&dbi, &cqi, now); !ok || err != nil {
+		t.Fatalf("ExecuteContinuousQuery failed, ok=%t, err=%v", ok, err)
+	}
+}
+
 // NewTestService returns a new *Service with default mock object members.
 func NewTestService(t *testing.T) *Service {
 	s := NewService(NewConfig())
 	ms := NewMetaClient(t)
 	s.MetaClient = ms
-	s.QueryExecutor = influxql.NewQueryExecutor()
+	s.QueryExecutor = query.NewQueryExecutor()
 	s.RunInterval = time.Millisecond
 
 	// Set Logger to write to dev/null so stdout isn't polluted.
-	if !testing.Verbose() {
-		s.Logger = log.New(ioutil.Discard, "", log.LstdFlags)
+	if testing.Verbose() {
+		s.WithLogger(logger.New(os.Stderr))
 	}
 
 	// Add a couple test databases and CQs.
@@ -395,7 +740,7 @@ func (ms *MetaClient) AcquireLease(name string) (l *meta.Lease, err error) {
 	return nil, meta.ErrServiceUnavailable
 }
 
-// Databases returns a list of database info about each database in the cluster.
+// Databases returns a list of database info about each database in the coordinator.
 func (ms *MetaClient) Databases() []meta.DatabaseInfo {
 	ms.mu.RLock()
 	defer ms.mu.RUnlock()
@@ -474,95 +819,15 @@ func (ms *MetaClient) CreateContinuousQuery(database, name, query string) error
 	return nil
 }
 
-// QueryExecutor is a mock query executor.
-type QueryExecutor struct {
-	*influxql.QueryExecutor
-	Err error
-	t   *testing.T
-}
-
 // StatementExecutor is a mock statement executor.
 type StatementExecutor struct {
-	ExecuteStatementFn func(stmt influxql.Statement, ctx *influxql.ExecutionContext) error
+	ExecuteStatementFn func(stmt influxql.Statement, ctx query.ExecutionContext) error
 }
 
-func (e *StatementExecutor) ExecuteStatement(stmt influxql.Statement, ctx *influxql.ExecutionContext) error {
+func (e *StatementExecutor) ExecuteStatement(stmt influxql.Statement, ctx query.ExecutionContext) error {
 	return e.ExecuteStatementFn(stmt, ctx)
 }
 
-func (e *StatementExecutor) NormalizeStatement(stmt influxql.Statement, database string) error {
-	return nil
-}
-
-// NewQueryExecutor returns a *QueryExecutor.
-func NewQueryExecutor(t *testing.T) *QueryExecutor {
-	e := influxql.NewQueryExecutor()
-	e.StatementExecutor = &StatementExecutor{}
-	return &QueryExecutor{
-		QueryExecutor: e,
-		t:             t,
-	}
-}
-
-// PointsWriter is a mock points writer.
-type PointsWriter struct {
-	WritePointsFn   func(p *cluster.WritePointsRequest) error
-	Err             error
-	PointsPerSecond int
-	t               *testing.T
-}
-
-// NewPointsWriter returns a new *PointsWriter.
-func NewPointsWriter(t *testing.T) *PointsWriter {
-	return &PointsWriter{
-		PointsPerSecond: 25000,
-		t:               t,
-	}
-}
-
-// WritePoints mocks writing points.
-func (pw *PointsWriter) WritePoints(p *cluster.WritePointsRequest) error {
-	// If the test set a callback, call it.
-	if pw.WritePointsFn != nil {
-		if err := pw.WritePointsFn(p); err != nil {
-			return err
-		}
-	}
-
-	if pw.Err != nil {
-		return pw.Err
-	}
-	ns := time.Duration((1 / pw.PointsPerSecond) * 1000000000)
-	time.Sleep(ns)
-	return nil
-}
-
-// genResult generates a dummy query result.
-func genResult(rowCnt, valCnt int) *influxql.Result {
-	rows := make(models.Rows, 0, rowCnt)
-	now := time.Now()
-	for n := 0; n < rowCnt; n++ {
-		vals := make([][]interface{}, 0, valCnt)
-		for m := 0; m < valCnt; m++ {
-			vals = append(vals, []interface{}{now, float64(m)})
-			now.Add(time.Second)
-		}
-		row := &models.Row{
-			Name:    "cpu",
-			Tags:    map[string]string{"host": "server01"},
-			Columns: []string{"time", "value"},
-			Values:  vals,
-		}
-		if len(rows) > 0 {
-			row.Name = fmt.Sprintf("cpu%d", len(rows)+1)
-		}
-		rows = append(rows, row)
-	}
-	return &influxql.Result{
-		Series: rows,
-	}
-}
-
 func wait(c chan struct{}, d time.Duration) (err error) {
 	select {
 	case <-c:
@@ -572,17 +837,18 @@ func wait(c chan struct{}, d time.Duration) (err error) {
 	return
 }
 
-func waitInt(c chan int, d time.Duration) (i int, err error) {
-	select {
-	case i = <-c:
-	case <-time.After(d):
-		err = errors.New("timed out")
-	}
-	return
+type monitor struct {
+	EnabledFn     func() bool
+	WritePointsFn func(models.Points) error
 }
 
-func check(err error) {
+func (m *monitor) Enabled() bool                     { return m.EnabledFn() }
+func (m *monitor) WritePoints(p models.Points) error { return m.WritePointsFn(p) }
+
+func mustParseTime(t *testing.T, value string) time.Time {
+	ts, err := time.Parse(time.RFC3339, value)
 	if err != nil {
-		panic(err)
+		t.Fatalf("unable to parse time: %s", err)
 	}
+	return ts
 }
