@@ -24,6 +24,7 @@ import (
 	"github.com/gogo/protobuf/proto"
 	"github.com/golang/snappy"
 	"github.com/influxdata/influxdb"
+	"github.com/influxdata/influxdb/logger"
 	"github.com/influxdata/influxdb/models"
 	"github.com/influxdata/influxdb/monitor"
 	"github.com/influxdata/influxdb/monitor/diagnostics"
@@ -34,7 +35,8 @@ import (
 	"github.com/influxdata/influxdb/tsdb"
 	"github.com/influxdata/influxdb/uuid"
 	"github.com/influxdata/influxql"
-	"github.com/uber-go/zap"
+	"github.com/prometheus/client_golang/prometheus/promhttp"
+	"go.uber.org/zap"
 )
 
 const (
@@ -107,8 +109,9 @@ type Handler struct {
 	}
 
 	Config    *Config
-	Logger    zap.Logger
+	Logger    *zap.Logger
 	CLFLogger *log.Logger
+	accessLog *os.File
 	stats     *Statistics
 
 	requestTracker *RequestTracker
@@ -119,7 +122,7 @@ func NewHandler(c Config) *Handler {
 	h := &Handler{
 		mux:            pat.New(),
 		Config:         &c,
-		Logger:         zap.New(zap.NullEncoder()),
+		Logger:         zap.NewNop(),
 		CLFLogger:      log.New(os.Stderr, "[httpd] ", 0),
 		stats:          &Statistics{},
 		requestTracker: NewRequestTracker(),
@@ -170,11 +173,40 @@ func NewHandler(c Config) *Handler {
 			"status-head",
 			"HEAD", "/status", false, true, h.serveStatus,
 		},
+		Route{
+			"prometheus-metrics",
+			"GET", "/metrics", false, true, promhttp.Handler().ServeHTTP,
+		},
 	}...)
 
 	return h
 }
 
+func (h *Handler) Open() {
+	if h.Config.LogEnabled {
+		path := "stderr"
+
+		if h.Config.AccessLogPath != "" {
+			f, err := os.OpenFile(h.Config.AccessLogPath, os.O_WRONLY|os.O_APPEND|os.O_CREATE, 0666)
+			if err != nil {
+				h.Logger.Error("unable to open access log, falling back to stderr", zap.Error(err), zap.String("path", h.Config.AccessLogPath))
+				return
+			}
+			h.CLFLogger = log.New(f, "", 0) // [httpd] prefix stripped when logging to a file
+			h.accessLog = f
+			path = h.Config.AccessLogPath
+		}
+		h.Logger.Info("opened HTTP access log", zap.String("path", path))
+	}
+}
+
+func (h *Handler) Close() {
+	if h.accessLog != nil {
+		h.accessLog.Close()
+		h.accessLog = nil
+	}
+}
+
 // Statistics maintains statistics for the httpd service.
 type Statistics struct {
 	Requests                     int64
@@ -387,7 +419,10 @@ func (h *Handler) serveQuery(w http.ResponseWriter, r *http.Request, user meta.U
 	if h.Config.AuthEnabled {
 		if err := h.QueryAuthorizer.AuthorizeQuery(user, q, db); err != nil {
 			if err, ok := err.(meta.ErrAuthorize); ok {
-				h.Logger.Info(fmt.Sprintf("Unauthorized request | user: %q | query: %q | database %q", err.User, err.Query.String(), err.Database))
+				h.Logger.Info("Unauthorized request",
+					zap.String("user", err.User),
+					zap.Stringer("query", err.Query),
+					logger.Database(err.Database))
 			}
 			h.httpError(rw, "error authorizing query: "+err.Error(), http.StatusForbidden)
 			return
@@ -593,7 +628,9 @@ func (h *Handler) async(q *influxql.Query, results <-chan *query.Result) {
 			if r.Err == query.ErrNotExecuted {
 				continue
 			}
-			h.Logger.Info(fmt.Sprintf("error while running async query: %s: %s", q, r.Err))
+			h.Logger.Info("Error while running async query",
+				zap.Stringer("query", q),
+				zap.Error(r.Err))
 		}
 	}
 }
@@ -676,7 +713,7 @@ func (h *Handler) serveWrite(w http.ResponseWriter, r *http.Request, user meta.U
 	atomic.AddInt64(&h.stats.WriteRequestBytesReceived, int64(buf.Len()))
 
 	if h.Config.WriteTracing {
-		h.Logger.Info(fmt.Sprintf("Write body received by handler: %s", buf.Bytes()))
+		h.Logger.Info("Write body received by handler", zap.ByteString("body", buf.Bytes()))
 	}
 
 	points, parseError := models.ParsePointsWithPrecision(buf.Bytes(), time.Now().UTC(), r.URL.Query().Get("precision"))
@@ -846,7 +883,7 @@ func (h *Handler) servePromWrite(w http.ResponseWriter, r *http.Request, user me
 	atomic.AddInt64(&h.stats.WriteRequestBytesReceived, int64(buf.Len()))
 
 	if h.Config.WriteTracing {
-		h.Logger.Info(fmt.Sprintf("Prom write body received by handler: %s", buf.Bytes()))
+		h.Logger.Info("Prom write body received by handler", zap.ByteString("body", buf.Bytes()))
 	}
 
 	reqBuf, err := snappy.Decode(nil, buf.Bytes())
@@ -865,7 +902,7 @@ func (h *Handler) servePromWrite(w http.ResponseWriter, r *http.Request, user me
 	points, err := prometheus.WriteRequestToPoints(&req)
 	if err != nil {
 		if h.Config.WriteTracing {
-			h.Logger.Info(fmt.Sprintf("Prom write handler: %s", err.Error()))
+			h.Logger.Info("Prom write handler", zap.Error(err))
 		}
 
 		if err != prometheus.ErrNaNDropped {
@@ -942,7 +979,10 @@ func (h *Handler) servePromRead(w http.ResponseWriter, r *http.Request, user met
 	if h.Config.AuthEnabled {
 		if err := h.QueryAuthorizer.AuthorizeQuery(user, q, db); err != nil {
 			if err, ok := err.(meta.ErrAuthorize); ok {
-				h.Logger.Info(fmt.Sprintf("Unauthorized request | user: %q | query: %q | database %q", err.User, err.Query.String(), err.Database))
+				h.Logger.Info("Unauthorized request",
+					zap.String("user", err.User),
+					zap.Stringer("query", err.Query),
+					logger.Database(err.Database))
 			}
 			h.httpError(w, "error authorizing query: "+err.Error(), http.StatusForbidden)
 			return
@@ -964,8 +1004,7 @@ func (h *Handler) servePromRead(w http.ResponseWriter, r *http.Request, user met
 	}
 
 	// Make sure if the client disconnects we signal the query to abort
-	var closing chan struct{}
-	closing = make(chan struct{})
+	closing := make(chan struct{})
 	if notifier, ok := w.(http.CloseNotifier); ok {
 		// CloseNotify() is not guaranteed to send a notification when the query
 		// is closed. Use this channel to signal that the query is finished to
