@@ -25,7 +25,7 @@ udp-payload-size = 1500
 	}
 
 	// Validate configuration.
-	if c.Enabled != true {
+	if !c.Enabled {
 		t.Fatalf("unexpected enabled: %v", c.Enabled)
 	} else if c.BindAddress != ":4444" {
 		t.Fatalf("unexpected bind address: %s", c.BindAddress)
