@@ -443,7 +443,7 @@ func TestCache_CacheSnapshot(t *testing.T) {
 	}
 
 	// Create another snapshot
-	snapshot, err = c.Snapshot()
+	_, err = c.Snapshot()
 	if err != nil {
 		t.Fatalf("failed to snapshot cache: %v", err)
 	}
@@ -454,7 +454,7 @@ func TestCache_CacheSnapshot(t *testing.T) {
 
 	c.ClearSnapshot(true)
 
-	snapshot, err = c.Snapshot()
+	_, err = c.Snapshot()
 	if err != nil {
 		t.Fatalf("failed to snapshot cache: %v", err)
 	}
@@ -956,7 +956,7 @@ func BenchmarkEntry_add(b *testing.B) {
 				otherValues[i] = NewValue(1, float64(i))
 			}
 
-			entry, err := newEntryValues(values, 0) // Will use default allocation size.
+			entry, err := newEntryValues(values)
 			if err != nil {
 				b.Fatal(err)
 			}
