@@ -1,15 +1,18 @@
 package tsm1_test
 
 import (
+	"context"
 	"fmt"
 	"io/ioutil"
 	"os"
 	"path/filepath"
 	"reflect"
 	"strings"
+	"sync/atomic"
 	"testing"
 	"time"
 
+	"github.com/influxdata/influxdb/logger"
 	"github.com/influxdata/influxdb/tsdb/engine/tsm1"
 )
 
@@ -30,10 +33,10 @@ func TestFileStore_Read(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	// Search for an entry that exists in the second file
-	values, err := fs.Read("cpu", 1)
+	values, err := fs.Read([]byte("cpu"), 1)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
 	}
@@ -67,10 +70,10 @@ func TestFileStore_SeekToAsc_FromStart(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	buf := make([]tsm1.FloatValue, 1000)
-	c := fs.KeyCursor("cpu", 0, true)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
 	// Search for an entry that exists in the second file
 	values, err := c.ReadFloatBlock(&buf)
 	if err != nil {
@@ -107,10 +110,10 @@ func TestFileStore_SeekToAsc_Duplicate(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	buf := make([]tsm1.FloatValue, 1000)
-	c := fs.KeyCursor("cpu", 0, true)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
 	// Search for an entry that exists in the second file
 	values, err := c.ReadFloatBlock(&buf)
 	if err != nil {
@@ -152,6 +155,10 @@ func TestFileStore_SeekToAsc_Duplicate(t *testing.T) {
 
 	c.Next()
 	values, err = c.ReadFloatBlock(&buf)
+	if err != nil {
+		t.Fatal(err)
+	}
+
 	exp = nil
 	if got, exp := len(values), len(exp); got != exp {
 		t.Fatalf("value length mismatch: got %v, exp %v", got, exp)
@@ -175,11 +182,11 @@ func TestFileStore_SeekToAsc_BeforeStart(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	// Search for an entry that exists in the second file
 	buf := make([]tsm1.FloatValue, 1000)
-	c := fs.KeyCursor("cpu", 0, true)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
 	values, err := c.ReadFloatBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -217,11 +224,11 @@ func TestFileStore_SeekToAsc_BeforeStart_OverlapFloat(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	// Search for an entry that exists in the second file
 	buf := make([]tsm1.FloatValue, 1000)
-	c := fs.KeyCursor("cpu", 0, true)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
 	values, err := c.ReadFloatBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -284,11 +291,11 @@ func TestFileStore_SeekToAsc_BeforeStart_OverlapInteger(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	// Search for an entry that exists in the second file
 	buf := make([]tsm1.IntegerValue, 1000)
-	c := fs.KeyCursor("cpu", 0, true)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
 	values, err := c.ReadIntegerBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -330,6 +337,72 @@ func TestFileStore_SeekToAsc_BeforeStart_OverlapInteger(t *testing.T) {
 	}
 }
 
+// Tests that seeking and reading all blocks that contain overlapping points does
+// not skip any blocks.
+func TestFileStore_SeekToAsc_BeforeStart_OverlapUnsigned(t *testing.T) {
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	fs := tsm1.NewFileStore(dir)
+
+	// Setup 3 files
+	data := []keyValues{
+		keyValues{"cpu", []tsm1.Value{tsm1.NewValue(0, uint64(0)), tsm1.NewValue(1, uint64(1))}},
+		keyValues{"cpu", []tsm1.Value{tsm1.NewValue(2, uint64(2))}},
+		keyValues{"cpu", []tsm1.Value{tsm1.NewValue(3, uint64(3))}},
+		keyValues{"cpu", []tsm1.Value{tsm1.NewValue(0, uint64(4)), tsm1.NewValue(2, uint64(7))}},
+	}
+
+	files, err := newFiles(dir, data...)
+	if err != nil {
+		t.Fatalf("unexpected error creating files: %v", err)
+	}
+
+	fs.Replace(nil, files)
+
+	// Search for an entry that exists in the second file
+	buf := make([]tsm1.UnsignedValue, 1000)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
+	values, err := c.ReadUnsignedBlock(&buf)
+	if err != nil {
+		t.Fatalf("unexpected error reading values: %v", err)
+	}
+
+	exp := []tsm1.Value{
+		data[3].values[0],
+		data[0].values[1],
+		data[3].values[1],
+	}
+	if got, exp := len(values), len(exp); got != exp {
+		t.Fatalf("value length mismatch: got %v, exp %v", got, exp)
+	}
+
+	for i, v := range exp {
+		if got, exp := values[i].Value(), v.Value(); got != exp {
+			t.Fatalf("read value mismatch(%d): got %v, exp %v", i, got, exp)
+		}
+	}
+
+	c.Next()
+	values, err = c.ReadUnsignedBlock(&buf)
+	if err != nil {
+		t.Fatalf("unexpected error reading values: %v", err)
+	}
+
+	exp = []tsm1.Value{
+		data[2].values[0],
+	}
+
+	if got, exp := len(values), len(exp); got != exp {
+		t.Fatalf("value length mismatch: got %v, exp %v", got, exp)
+	}
+
+	for i, v := range exp {
+		if got, exp := values[i].Value(), v.Value(); got != exp {
+			t.Fatalf("read value mismatch(%d): got %v, exp %v", i, got, exp)
+		}
+	}
+}
+
 // Tests that seeking and reading all blocks that contain overlapping points does
 // not skip any blocks.
 func TestFileStore_SeekToAsc_BeforeStart_OverlapBoolean(t *testing.T) {
@@ -350,11 +423,11 @@ func TestFileStore_SeekToAsc_BeforeStart_OverlapBoolean(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	// Search for an entry that exists in the second file
 	buf := make([]tsm1.BooleanValue, 1000)
-	c := fs.KeyCursor("cpu", 0, true)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
 	values, err := c.ReadBooleanBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -416,11 +489,11 @@ func TestFileStore_SeekToAsc_BeforeStart_OverlapString(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	// Search for an entry that exists in the second file
 	buf := make([]tsm1.StringValue, 1000)
-	c := fs.KeyCursor("cpu", 0, true)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
 	values, err := c.ReadStringBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -482,10 +555,10 @@ func TestFileStore_SeekToAsc_OverlapMinFloat(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	buf := make([]tsm1.FloatValue, 1000)
-	c := fs.KeyCursor("cpu", 0, true)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
 	// Search for an entry that exists in the second file
 	values, err := c.ReadFloatBlock(&buf)
 	if err != nil {
@@ -532,6 +605,10 @@ func TestFileStore_SeekToAsc_OverlapMinFloat(t *testing.T) {
 
 	c.Next()
 	values, err = c.ReadFloatBlock(&buf)
+	if err != nil {
+		t.Fatal(err)
+	}
+
 	exp = nil
 	if got, exp := len(values), len(exp); got != exp {
 		t.Fatalf("value length mismatch: got %v, exp %v", got, exp)
@@ -558,10 +635,10 @@ func TestFileStore_SeekToAsc_OverlapMinInteger(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	buf := make([]tsm1.IntegerValue, 1000)
-	c := fs.KeyCursor("cpu", 0, true)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
 	// Search for an entry that exists in the second file
 	values, err := c.ReadIntegerBlock(&buf)
 	if err != nil {
@@ -607,6 +684,89 @@ func TestFileStore_SeekToAsc_OverlapMinInteger(t *testing.T) {
 
 	c.Next()
 	values, err = c.ReadIntegerBlock(&buf)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	exp = nil
+	if got, exp := len(values), len(exp); got != exp {
+		t.Fatalf("value length mismatch: got %v, exp %v", got, exp)
+	}
+}
+
+// Tests that blocks with a lower min time in later files are not returned
+// more than once causing unsorted results
+func TestFileStore_SeekToAsc_OverlapMinUnsigned(t *testing.T) {
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	fs := tsm1.NewFileStore(dir)
+
+	// Setup 3 files
+	data := []keyValues{
+		keyValues{"cpu", []tsm1.Value{tsm1.NewValue(1, uint64(1)), tsm1.NewValue(3, uint64(3))}},
+		keyValues{"cpu", []tsm1.Value{tsm1.NewValue(2, uint64(2)), tsm1.NewValue(4, uint64(4))}},
+		keyValues{"cpu", []tsm1.Value{tsm1.NewValue(0, uint64(0)), tsm1.NewValue(1, uint64(10))}},
+		keyValues{"cpu", []tsm1.Value{tsm1.NewValue(2, uint64(5))}},
+	}
+
+	files, err := newFiles(dir, data...)
+	if err != nil {
+		t.Fatalf("unexpected error creating files: %v", err)
+	}
+
+	fs.Replace(nil, files)
+
+	buf := make([]tsm1.UnsignedValue, 1000)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
+	// Search for an entry that exists in the second file
+	values, err := c.ReadUnsignedBlock(&buf)
+	if err != nil {
+		t.Fatalf("unexpected error reading values: %v", err)
+	}
+
+	exp := []tsm1.Value{
+		data[2].values[0],
+		data[2].values[1],
+		data[3].values[0],
+		data[0].values[1],
+	}
+
+	if got, exp := len(values), len(exp); got != exp {
+		t.Fatalf("value length mismatch: got %v, exp %v", got, exp)
+	}
+
+	for i, v := range exp {
+		if got, exp := values[i].Value(), v.Value(); got != exp {
+			t.Fatalf("read value mismatch(%d): got %v, exp %v", i, got, exp)
+		}
+	}
+
+	// Check that calling Next will dedupe points
+	c.Next()
+	values, err = c.ReadUnsignedBlock(&buf)
+	if err != nil {
+		t.Fatalf("unexpected error reading values: %v", err)
+	}
+
+	exp = []tsm1.Value{
+		data[1].values[1],
+	}
+	if got, exp := len(values), len(exp); got != exp {
+		t.Fatalf("value length mismatch: got %v, exp %v", got, exp)
+	}
+
+	for i, v := range exp {
+		if got, exp := values[i].Value(), v.Value(); got != exp {
+			t.Fatalf("read value mismatch(%d): got %v, exp %v", i, got, exp)
+		}
+	}
+
+	c.Next()
+	values, err = c.ReadUnsignedBlock(&buf)
+	if err != nil {
+		t.Fatal(err)
+	}
+
 	exp = nil
 	if got, exp := len(values), len(exp); got != exp {
 		t.Fatalf("value length mismatch: got %v, exp %v", got, exp)
@@ -633,10 +793,10 @@ func TestFileStore_SeekToAsc_OverlapMinBoolean(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	buf := make([]tsm1.BooleanValue, 1000)
-	c := fs.KeyCursor("cpu", 0, true)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
 	// Search for an entry that exists in the second file
 	values, err := c.ReadBooleanBlock(&buf)
 	if err != nil {
@@ -682,6 +842,10 @@ func TestFileStore_SeekToAsc_OverlapMinBoolean(t *testing.T) {
 
 	c.Next()
 	values, err = c.ReadBooleanBlock(&buf)
+	if err != nil {
+		t.Fatal(err)
+	}
+
 	exp = nil
 	if got, exp := len(values), len(exp); got != exp {
 		t.Fatalf("value length mismatch: got %v, exp %v", got, exp)
@@ -708,10 +872,10 @@ func TestFileStore_SeekToAsc_OverlapMinString(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	buf := make([]tsm1.StringValue, 1000)
-	c := fs.KeyCursor("cpu", 0, true)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
 	// Search for an entry that exists in the second file
 	values, err := c.ReadStringBlock(&buf)
 	if err != nil {
@@ -757,6 +921,10 @@ func TestFileStore_SeekToAsc_OverlapMinString(t *testing.T) {
 
 	c.Next()
 	values, err = c.ReadStringBlock(&buf)
+	if err != nil {
+		t.Fatal(err)
+	}
+
 	exp = nil
 	if got, exp := len(values), len(exp); got != exp {
 		t.Fatalf("value length mismatch: got %v, exp %v", got, exp)
@@ -781,11 +949,11 @@ func TestFileStore_SeekToAsc_Middle(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	// Search for an entry that exists in the second file
 	buf := make([]tsm1.FloatValue, 1000)
-	c := fs.KeyCursor("cpu", 3, true)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 3, true)
 	values, err := c.ReadFloatBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -838,10 +1006,10 @@ func TestFileStore_SeekToAsc_End(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	buf := make([]tsm1.FloatValue, 1000)
-	c := fs.KeyCursor("cpu", 2, true)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 2, true)
 	values, err := c.ReadFloatBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -876,11 +1044,11 @@ func TestFileStore_SeekToDesc_FromStart(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	// Search for an entry that exists in the second file
 	buf := make([]tsm1.FloatValue, 1000)
-	c := fs.KeyCursor("cpu", 0, false)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, false)
 	values, err := c.ReadFloatBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -915,11 +1083,11 @@ func TestFileStore_SeekToDesc_Duplicate(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	// Search for an entry that exists in the second file
 	buf := make([]tsm1.FloatValue, 1000)
-	c := fs.KeyCursor("cpu", 2, false)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 2, false)
 	values, err := c.ReadFloatBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -974,11 +1142,11 @@ func TestFileStore_SeekToDesc_OverlapMaxFloat(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	// Search for an entry that exists in the second file
 	buf := make([]tsm1.FloatValue, 1000)
-	c := fs.KeyCursor("cpu", 5, false)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 5, false)
 	values, err := c.ReadFloatBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -1039,11 +1207,11 @@ func TestFileStore_SeekToDesc_OverlapMaxInteger(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	// Search for an entry that exists in the second file
 	buf := make([]tsm1.IntegerValue, 1000)
-	c := fs.KeyCursor("cpu", 5, false)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 5, false)
 	values, err := c.ReadIntegerBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -1083,6 +1251,68 @@ func TestFileStore_SeekToDesc_OverlapMaxInteger(t *testing.T) {
 		}
 	}
 }
+func TestFileStore_SeekToDesc_OverlapMaxUnsigned(t *testing.T) {
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	fs := tsm1.NewFileStore(dir)
+
+	// Setup 3 files
+	data := []keyValues{
+		keyValues{"cpu", []tsm1.Value{tsm1.NewValue(1, uint64(1)), tsm1.NewValue(3, uint64(3))}},
+		keyValues{"cpu", []tsm1.Value{tsm1.NewValue(2, uint64(2)), tsm1.NewValue(4, uint64(4))}},
+		keyValues{"cpu", []tsm1.Value{tsm1.NewValue(0, uint64(0)), tsm1.NewValue(1, uint64(10))}},
+		keyValues{"cpu", []tsm1.Value{tsm1.NewValue(2, uint64(5))}},
+	}
+
+	files, err := newFiles(dir, data...)
+	if err != nil {
+		t.Fatalf("unexpected error creating files: %v", err)
+	}
+
+	fs.Replace(nil, files)
+
+	// Search for an entry that exists in the second file
+	buf := make([]tsm1.UnsignedValue, 1000)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 5, false)
+	values, err := c.ReadUnsignedBlock(&buf)
+	if err != nil {
+		t.Fatalf("unexpected error reading values: %v", err)
+	}
+
+	exp := []tsm1.Value{
+		data[3].values[0],
+		data[0].values[1],
+		data[1].values[1],
+	}
+	if got, exp := len(values), len(exp); got != exp {
+		t.Fatalf("value length mismatch: got %v, exp %v", got, exp)
+	}
+
+	for i, v := range exp {
+		if got, exp := values[i].Value(), v.Value(); got != exp {
+			t.Fatalf("read value mismatch(%d): got %v, exp %v", i, got, exp)
+		}
+	}
+
+	c.Next()
+	values, err = c.ReadUnsignedBlock(&buf)
+	if err != nil {
+		t.Fatalf("unexpected error reading values: %v", err)
+	}
+	exp = []tsm1.Value{
+		data[2].values[0],
+		data[2].values[1],
+	}
+	if got, exp := len(values), len(exp); got != exp {
+		t.Fatalf("value length mismatch: got %v, exp %v", got, exp)
+	}
+
+	for i, v := range exp {
+		if got, exp := values[i].Value(), v.Value(); got != exp {
+			t.Fatalf("read value mismatch(%d): got %v, exp %v", i, got, exp)
+		}
+	}
+}
 
 func TestFileStore_SeekToDesc_OverlapMaxBoolean(t *testing.T) {
 	dir := MustTempDir()
@@ -1102,11 +1332,11 @@ func TestFileStore_SeekToDesc_OverlapMaxBoolean(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	// Search for an entry that exists in the second file
 	buf := make([]tsm1.BooleanValue, 1000)
-	c := fs.KeyCursor("cpu", 5, false)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 5, false)
 	values, err := c.ReadBooleanBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -1165,11 +1395,11 @@ func TestFileStore_SeekToDesc_OverlapMaxString(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	// Search for an entry that exists in the second file
 	buf := make([]tsm1.StringValue, 1000)
-	c := fs.KeyCursor("cpu", 5, false)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 5, false)
 	values, err := c.ReadStringBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -1227,10 +1457,10 @@ func TestFileStore_SeekToDesc_AfterEnd(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	buf := make([]tsm1.FloatValue, 1000)
-	c := fs.KeyCursor("cpu", 4, false)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 4, false)
 	values, err := c.ReadFloatBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -1266,10 +1496,10 @@ func TestFileStore_SeekToDesc_AfterEnd_OverlapFloat(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	buf := make([]tsm1.FloatValue, 1000)
-	c := fs.KeyCursor("cpu", 10, false)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 10, false)
 	values, err := c.ReadFloatBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -1363,10 +1593,10 @@ func TestFileStore_SeekToDesc_AfterEnd_OverlapInteger(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	buf := make([]tsm1.IntegerValue, 1000)
-	c := fs.KeyCursor("cpu", 11, false)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 11, false)
 	values, err := c.ReadIntegerBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -1422,6 +1652,83 @@ func TestFileStore_SeekToDesc_AfterEnd_OverlapInteger(t *testing.T) {
 	}
 }
 
+func TestFileStore_SeekToDesc_AfterEnd_OverlapUnsigned(t *testing.T) {
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	fs := tsm1.NewFileStore(dir)
+
+	// Setup 3 files
+	data := []keyValues{
+		keyValues{"cpu", []tsm1.Value{tsm1.NewValue(8, uint64(0)), tsm1.NewValue(9, uint64(1))}},
+		keyValues{"cpu", []tsm1.Value{tsm1.NewValue(2, uint64(2))}},
+		keyValues{"cpu", []tsm1.Value{tsm1.NewValue(3, uint64(3))}},
+		keyValues{"cpu", []tsm1.Value{tsm1.NewValue(3, uint64(4)), tsm1.NewValue(10, uint64(7))}},
+	}
+
+	files, err := newFiles(dir, data...)
+	if err != nil {
+		t.Fatalf("unexpected error creating files: %v", err)
+	}
+
+	fs.Replace(nil, files)
+
+	buf := make([]tsm1.UnsignedValue, 1000)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 11, false)
+	values, err := c.ReadUnsignedBlock(&buf)
+	if err != nil {
+		t.Fatalf("unexpected error reading values: %v", err)
+	}
+
+	exp := []tsm1.Value{
+		data[3].values[0],
+		data[0].values[0],
+		data[0].values[1],
+		data[3].values[1],
+	}
+
+	if got, exp := len(values), len(exp); got != exp {
+		t.Fatalf("value length mismatch: got %v, exp %v", got, exp)
+	}
+
+	for i, v := range exp {
+		if got, exp := values[i].Value(), v.Value(); got != exp {
+			t.Fatalf("read value mismatch(%d): got %v, exp %v", i, got, exp)
+		}
+	}
+
+	c.Next()
+	values, err = c.ReadUnsignedBlock(&buf)
+
+	if err != nil {
+		t.Fatalf("unexpected error reading values: %v", err)
+	}
+
+	exp = []tsm1.Value{
+		data[1].values[0],
+	}
+
+	if got, exp := len(values), len(exp); got != exp {
+		t.Fatalf("value length mismatch: got %v, exp %v", got, exp)
+	}
+
+	for i, v := range exp {
+		if got, exp := values[i].Value(), v.Value(); got != exp {
+			t.Fatalf("read value mismatch(%d): got %v, exp %v", i, got, exp)
+		}
+	}
+
+	c.Next()
+	values, err = c.ReadUnsignedBlock(&buf)
+
+	if err != nil {
+		t.Fatalf("unexpected error reading values: %v", err)
+	}
+
+	if got, exp := len(values), 0; got != exp {
+		t.Fatalf("value length mismatch: got %v, exp %v", got, exp)
+	}
+}
+
 func TestFileStore_SeekToDesc_AfterEnd_OverlapBoolean(t *testing.T) {
 	dir := MustTempDir()
 	defer os.RemoveAll(dir)
@@ -1440,10 +1747,10 @@ func TestFileStore_SeekToDesc_AfterEnd_OverlapBoolean(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	buf := make([]tsm1.BooleanValue, 1000)
-	c := fs.KeyCursor("cpu", 11, false)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 11, false)
 	values, err := c.ReadBooleanBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -1537,10 +1844,10 @@ func TestFileStore_SeekToDesc_AfterEnd_OverlapString(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	buf := make([]tsm1.StringValue, 1000)
-	c := fs.KeyCursor("cpu", 11, false)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 11, false)
 	values, err := c.ReadStringBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -1636,11 +1943,11 @@ func TestFileStore_SeekToDesc_Middle(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	// Search for an entry that exists in the second file
 	buf := make([]tsm1.FloatValue, 1000)
-	c := fs.KeyCursor("cpu", 3, false)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 3, false)
 	values, err := c.ReadFloatBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -1710,10 +2017,10 @@ func TestFileStore_SeekToDesc_End(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	buf := make([]tsm1.FloatValue, 1000)
-	c := fs.KeyCursor("cpu", 2, false)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 2, false)
 	values, err := c.ReadFloatBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -1748,14 +2055,14 @@ func TestKeyCursor_TombstoneRange(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
-	if err := fs.DeleteRange([]string{"cpu"}, 1, 1); err != nil {
+	if err := fs.DeleteRange([][]byte{[]byte("cpu")}, 1, 1); err != nil {
 		t.Fatalf("unexpected error delete range: %v", err)
 	}
 
 	buf := make([]tsm1.FloatValue, 1000)
-	c := fs.KeyCursor("cpu", 0, true)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
 	expValues := []int{0, 2}
 	for _, v := range expValues {
 		values, err := c.ReadFloatBlock(&buf)
@@ -1775,6 +2082,49 @@ func TestKeyCursor_TombstoneRange(t *testing.T) {
 	}
 }
 
+func TestKeyCursor_TombstoneRange_PartialFirst(t *testing.T) {
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	fs := tsm1.NewFileStore(dir)
+
+	// Setup 3 files
+	data := []keyValues{
+		keyValues{"cpu", []tsm1.Value{tsm1.NewValue(0, 0.0), tsm1.NewValue(1, 1.0)}},
+		keyValues{"cpu", []tsm1.Value{tsm1.NewValue(2, 2.0)}},
+	}
+
+	files, err := newFiles(dir, data...)
+	if err != nil {
+		t.Fatalf("unexpected error creating files: %v", err)
+	}
+
+	// Delete part of the block in the first file.
+	r := MustOpenTSMReader(files[0])
+	r.DeleteRange([][]byte{[]byte("cpu")}, 1, 3)
+
+	fs.Replace(nil, files)
+
+	buf := make([]tsm1.FloatValue, 1000)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
+	expValues := []tsm1.Value{tsm1.NewValue(0, 0.0), tsm1.NewValue(2, 2.0)}
+
+	for _, exp := range expValues {
+		values, err := c.ReadFloatBlock(&buf)
+		if err != nil {
+			t.Fatalf("unexpected error reading values: %v", err)
+		}
+
+		if got, exp := len(values), 1; got != exp {
+			t.Fatalf("value length mismatch: got %v, exp %v", got, exp)
+		}
+
+		if got, exp := values[0].String(), exp.String(); got != exp {
+			t.Fatalf("read value mismatch(%d): got %v, exp %v", 0, got, exp)
+		}
+		c.Next()
+	}
+}
+
 func TestKeyCursor_TombstoneRange_PartialFloat(t *testing.T) {
 	dir := MustTempDir()
 	defer os.RemoveAll(dir)
@@ -1793,14 +2143,14 @@ func TestKeyCursor_TombstoneRange_PartialFloat(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
-	if err := fs.DeleteRange([]string{"cpu"}, 1, 1); err != nil {
+	if err := fs.DeleteRange([][]byte{[]byte("cpu")}, 1, 1); err != nil {
 		t.Fatalf("unexpected error delete range: %v", err)
 	}
 
 	buf := make([]tsm1.FloatValue, 1000)
-	c := fs.KeyCursor("cpu", 0, true)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
 	values, err := c.ReadFloatBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -1837,14 +2187,14 @@ func TestKeyCursor_TombstoneRange_PartialInteger(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
-	if err := fs.DeleteRange([]string{"cpu"}, 1, 1); err != nil {
+	if err := fs.DeleteRange([][]byte{[]byte("cpu")}, 1, 1); err != nil {
 		t.Fatalf("unexpected error delete range: %v", err)
 	}
 
 	buf := make([]tsm1.IntegerValue, 1000)
-	c := fs.KeyCursor("cpu", 0, true)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
 	values, err := c.ReadIntegerBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -1863,6 +2213,50 @@ func TestKeyCursor_TombstoneRange_PartialInteger(t *testing.T) {
 	}
 }
 
+func TestKeyCursor_TombstoneRange_PartialUnsigned(t *testing.T) {
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	fs := tsm1.NewFileStore(dir)
+
+	// Setup 3 files
+	data := []keyValues{
+		keyValues{"cpu", []tsm1.Value{
+			tsm1.NewValue(0, uint64(1)),
+			tsm1.NewValue(1, uint64(2)),
+			tsm1.NewValue(2, uint64(3))}},
+	}
+
+	files, err := newFiles(dir, data...)
+	if err != nil {
+		t.Fatalf("unexpected error creating files: %v", err)
+	}
+
+	fs.Replace(nil, files)
+
+	if err := fs.DeleteRange([][]byte{[]byte("cpu")}, 1, 1); err != nil {
+		t.Fatalf("unexpected error delete range: %v", err)
+	}
+
+	buf := make([]tsm1.UnsignedValue, 1000)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
+	values, err := c.ReadUnsignedBlock(&buf)
+	if err != nil {
+		t.Fatalf("unexpected error reading values: %v", err)
+	}
+
+	expValues := []tsm1.Value{data[0].values[0], data[0].values[2]}
+	for i, v := range expValues {
+		exp := v
+		if got, exp := len(values), 2; got != exp {
+			t.Fatalf("value length mismatch: got %v, exp %v", got, exp)
+		}
+
+		if got, exp := values[i].String(), exp.String(); got != exp {
+			t.Fatalf("read value mismatch(%d): got %v, exp %v", 0, got, exp)
+		}
+	}
+}
+
 func TestKeyCursor_TombstoneRange_PartialString(t *testing.T) {
 	dir := MustTempDir()
 	defer os.RemoveAll(dir)
@@ -1881,14 +2275,14 @@ func TestKeyCursor_TombstoneRange_PartialString(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
-	if err := fs.DeleteRange([]string{"cpu"}, 1, 1); err != nil {
+	if err := fs.DeleteRange([][]byte{[]byte("cpu")}, 1, 1); err != nil {
 		t.Fatalf("unexpected error delete range: %v", err)
 	}
 
 	buf := make([]tsm1.StringValue, 1000)
-	c := fs.KeyCursor("cpu", 0, true)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
 	values, err := c.ReadStringBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -1925,14 +2319,14 @@ func TestKeyCursor_TombstoneRange_PartialBoolean(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
-	if err := fs.DeleteRange([]string{"cpu"}, 1, 1); err != nil {
+	if err := fs.DeleteRange([][]byte{[]byte("cpu")}, 1, 1); err != nil {
 		t.Fatalf("unexpected error delete range: %v", err)
 	}
 
 	buf := make([]tsm1.BooleanValue, 1000)
-	c := fs.KeyCursor("cpu", 0, true)
+	c := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
 	values, err := c.ReadBooleanBlock(&buf)
 	if err != nil {
 		t.Fatalf("unexpected error reading values: %v", err)
@@ -2012,7 +2406,7 @@ func TestFileStore_Remove(t *testing.T) {
 		t.Fatalf("current ID mismatch: got %v, exp %v", got, exp)
 	}
 
-	fs.Remove(files[2])
+	fs.Replace(files[2:3], nil)
 
 	if got, exp := fs.Count(), 2; got != exp {
 		t.Fatalf("file count mismatch: got %v, exp %v", got, exp)
@@ -2040,7 +2434,7 @@ func TestFileStore_Replace(t *testing.T) {
 	}
 
 	// Replace requires assumes new files have a .tmp extension
-	replacement := files[2] + ".tmp"
+	replacement := fmt.Sprintf("%s.%s", files[2], tsm1.TmpTSMFileExtension)
 	os.Rename(files[2], replacement)
 
 	fs := tsm1.NewFileStore(dir)
@@ -2054,7 +2448,7 @@ func TestFileStore_Replace(t *testing.T) {
 	}
 
 	// Should record references to the two existing TSM files
-	cur := fs.KeyCursor("cpu", 0, true)
+	cur := fs.KeyCursor(context.Background(), []byte("cpu"), 0, true)
 
 	// Should move the existing files out of the way, but allow query to complete
 	if err := fs.Replace(files[:2], []string{replacement}); err != nil {
@@ -2069,11 +2463,18 @@ func TestFileStore_Replace(t *testing.T) {
 	cur.Next()
 	buf := make([]tsm1.FloatValue, 10)
 	values, err := cur.ReadFloatBlock(&buf)
+	if err != nil {
+		t.Fatal(err)
+	}
 	if got, exp := len(values), 1; got != exp {
 		t.Fatalf("value len mismatch: got %v, exp %v", got, exp)
 	}
+
 	cur.Next()
 	values, err = cur.ReadFloatBlock(&buf)
+	if err != nil {
+		t.Fatal(err)
+	}
 	if got, exp := len(values), 1; got != exp {
 		t.Fatalf("value len mismatch: got %v, exp %v", got, exp)
 	}
@@ -2081,6 +2482,9 @@ func TestFileStore_Replace(t *testing.T) {
 	// No more blocks for this cursor
 	cur.Next()
 	values, err = cur.ReadFloatBlock(&buf)
+	if err != nil {
+		t.Fatal(err)
+	}
 	if got, exp := len(values), 0; got != exp {
 		t.Fatalf("value len mismatch: got %v, exp %v", got, exp)
 	}
@@ -2130,7 +2534,7 @@ func TestFileStore_Open_Deleted(t *testing.T) {
 		t.Fatalf("file count mismatch: got %v, exp %v", got, exp)
 	}
 
-	if err := fs.Delete([]string{"cpu,host=server2!~#!value"}); err != nil {
+	if err := fs.Delete([][]byte{[]byte("cpu,host=server2!~#!value")}); err != nil {
 		fatal(t, "deleting", err)
 	}
 
@@ -2162,14 +2566,14 @@ func TestFileStore_Delete(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	keys := fs.Keys()
 	if got, exp := len(keys), 3; got != exp {
 		t.Fatalf("key length mismatch: got %v, exp %v", got, exp)
 	}
 
-	if err := fs.Delete([]string{"cpu,host=server2!~#!value"}); err != nil {
+	if err := fs.Delete([][]byte{[]byte("cpu,host=server2!~#!value")}); err != nil {
 		fatal(t, "deleting", err)
 	}
 
@@ -2179,6 +2583,43 @@ func TestFileStore_Delete(t *testing.T) {
 	}
 }
 
+func TestFileStore_Apply(t *testing.T) {
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	fs := tsm1.NewFileStore(dir)
+
+	// Setup 3 files
+	data := []keyValues{
+		keyValues{"cpu,host=server2#!~#value", []tsm1.Value{tsm1.NewValue(0, 1.0)}},
+		keyValues{"cpu,host=server1#!~#value", []tsm1.Value{tsm1.NewValue(1, 2.0)}},
+		keyValues{"mem,host=server1#!~#value", []tsm1.Value{tsm1.NewValue(0, 1.0)}},
+	}
+
+	files, err := newFiles(dir, data...)
+	if err != nil {
+		t.Fatalf("unexpected error creating files: %v", err)
+	}
+
+	fs.Replace(nil, files)
+
+	keys := fs.Keys()
+	if got, exp := len(keys), 3; got != exp {
+		t.Fatalf("key length mismatch: got %v, exp %v", got, exp)
+	}
+
+	var n int64
+	if err := fs.Apply(func(r tsm1.TSMFile) error {
+		atomic.AddInt64(&n, 1)
+		return nil
+	}); err != nil {
+		t.Fatalf("unexpected error deleting: %v", err)
+	}
+
+	if got, exp := n, int64(3); got != exp {
+		t.Fatalf("apply mismatch: got %v, exp %v", got, exp)
+	}
+}
+
 func TestFileStore_Stats(t *testing.T) {
 	dir := MustTempDir()
 	defer os.RemoveAll(dir)
@@ -2212,7 +2653,7 @@ func TestFileStore_Stats(t *testing.T) {
 	}
 
 	// Removing one of the files should invalidate the cache.
-	fs.Remove(files[0])
+	fs.Replace(files[0:1], nil)
 	if got, exp := len(fs.Stats()), 2; got != exp {
 		t.Fatalf("file count mismatch: got %v, exp %v", got, exp)
 	}
@@ -2222,9 +2663,9 @@ func TestFileStore_Stats(t *testing.T) {
 		"mem": []tsm1.Value{tsm1.NewValue(0, 1.0)},
 	})
 
-	replacement := files[2] + "-foo" + ".tmp" // Assumes new files have a .tmp extension
+	replacement := fmt.Sprintf("%s.%s.%s", files[2], tsm1.TmpTSMFileExtension, tsm1.TSMFileExtension) // Assumes new files have a .tmp extension
 	if err := os.Rename(newFile, replacement); err != nil {
-
+		t.Fatalf("rename: %v", err)
 	}
 	// Replace 3 w/ 1
 	if err := fs.Replace(files, []string{replacement}); err != nil {
@@ -2234,7 +2675,7 @@ func TestFileStore_Stats(t *testing.T) {
 	var found bool
 	stats = fs.Stats()
 	for _, stat := range stats {
-		if strings.HasSuffix(stat.Path, "-foo") {
+		if strings.HasSuffix(stat.Path, fmt.Sprintf("%s.%s.%s", tsm1.TSMFileExtension, tsm1.TmpTSMFileExtension, tsm1.TSMFileExtension)) {
 			found = true
 		}
 	}
@@ -2247,18 +2688,8 @@ func TestFileStore_Stats(t *testing.T) {
 		"mem": []tsm1.Value{tsm1.NewValue(0, 1.0)},
 	})
 
-	fd, err := os.Open(newFile)
-	if err != nil {
-		t.Fatalf("open file: %v", err)
-	}
-
-	f, err := tsm1.NewTSMReader(fd)
-	if err != nil {
-		t.Fatalf("new reader: %v", err)
-	}
-
 	// Adding some files should invalidate the cache.
-	fs.Add(f)
+	fs.Replace(nil, []string{newFile})
 	if got, exp := len(fs.Stats()), 2; got != exp {
 		t.Fatalf("file count mismatch: got %v, exp %v", got, exp)
 	}
@@ -2281,10 +2712,10 @@ func TestFileStore_CreateSnapshot(t *testing.T) {
 		t.Fatalf("unexpected error creating files: %v", err)
 	}
 
-	fs.Add(files...)
+	fs.Replace(nil, files)
 
 	// Create a tombstone
-	if err := fs.DeleteRange([]string{"cpu"}, 1, 1); err != nil {
+	if err := fs.DeleteRange([][]byte{[]byte("cpu")}, 1, 1); err != nil {
 		t.Fatalf("unexpected error delete range: %v", err)
 	}
 
@@ -2329,7 +2760,7 @@ func newFileDir(dir string, values ...keyValues) ([]string, error) {
 			return nil, err
 		}
 
-		if err := w.Write(v.key, v.values); err != nil {
+		if err := w.Write([]byte(v.key), v.values); err != nil {
 			return nil, err
 		}
 
@@ -2352,8 +2783,8 @@ func newFileDir(dir string, values ...keyValues) ([]string, error) {
 
 }
 
-func newFiles(dir string, values ...keyValues) ([]tsm1.TSMFile, error) {
-	var files []tsm1.TSMFile
+func newFiles(dir string, values ...keyValues) ([]string, error) {
+	var files []string
 
 	id := 1
 	for _, v := range values {
@@ -2363,7 +2794,7 @@ func newFiles(dir string, values ...keyValues) ([]tsm1.TSMFile, error) {
 			return nil, err
 		}
 
-		if err := w.Write(v.key, v.values); err != nil {
+		if err := w.Write([]byte(v.key), v.values); err != nil {
 			return nil, err
 		}
 
@@ -2381,15 +2812,7 @@ func newFiles(dir string, values ...keyValues) ([]tsm1.TSMFile, error) {
 		}
 		id++
 
-		fd, err := os.Open(newName)
-		if err != nil {
-			return nil, err
-		}
-		r, err := tsm1.NewTSMReader(fd)
-		if err != nil {
-			return nil, err
-		}
-		files = append(files, r)
+		files = append(files, newName)
 	}
 	return files, nil
 }
@@ -2441,8 +2864,8 @@ func BenchmarkFileStore_Stats(b *testing.B) {
 	}
 
 	fs := tsm1.NewFileStore(dir)
-	if !testing.Verbose() {
-		fs.SetLogOutput(ioutil.Discard)
+	if testing.Verbose() {
+		fs.WithLogger(logger.New(os.Stderr))
 	}
 
 	if err := fs.Open(); err != nil {
