@@ -14,23 +14,18 @@ import (
 	"fmt"
 	"math"
 
-	"github.com/dgryski/go-bits"
 	"github.com/dgryski/go-bitstream"
+	"github.com/influxdata/influxdb/pkg/bits"
 )
 
-const (
-	// floatUncompressed is an uncompressed format using 8 bytes per value.
-	// Not yet implemented.
-	floatUncompressed = 0
-
-	// floatCompressedGorilla is a compressed format using the gorilla paper encoding
-	floatCompressedGorilla = 1
-)
+// Note: an uncompressed format is not yet implemented.
+// floatCompressedGorilla is a compressed format using the gorilla paper encoding
+const floatCompressedGorilla = 1
 
 // uvnan is the constant returned from math.NaN().
 const uvnan = 0x7FF8000000000001
 
-// FloatEncoder encodes multiple float64s into a byte slice
+// FloatEncoder encodes multiple float64s into a byte slice.
 type FloatEncoder struct {
 	val float64
 	err error
@@ -45,6 +40,7 @@ type FloatEncoder struct {
 	finished bool
 }
 
+// NewFloatEncoder returns a new FloatEncoder.
 func NewFloatEncoder() *FloatEncoder {
 	s := FloatEncoder{
 		first:   true,
@@ -55,9 +51,9 @@ func NewFloatEncoder() *FloatEncoder {
 	s.buf.WriteByte(floatCompressedGorilla << 4)
 
 	return &s
-
 }
 
+// Reset sets the encoder back to its initial state.
 func (s *FloatEncoder) Reset() {
 	s.val = 0
 	s.err = nil
@@ -72,20 +68,23 @@ func (s *FloatEncoder) Reset() {
 	s.first = true
 }
 
+// Bytes returns a copy of the underlying byte buffer used in the encoder.
 func (s *FloatEncoder) Bytes() ([]byte, error) {
 	return s.buf.Bytes(), s.err
 }
 
-func (s *FloatEncoder) Finish() {
+// Flush indicates there are no more values to encode.
+func (s *FloatEncoder) Flush() {
 	if !s.finished {
 		// write an end-of-stream record
 		s.finished = true
-		s.Push(math.NaN())
+		s.Write(math.NaN())
 		s.bw.Flush(bitstream.Zero)
 	}
 }
 
-func (s *FloatEncoder) Push(v float64) {
+// Write encodes v to the underlying buffer.
+func (s *FloatEncoder) Write(v float64) {
 	// Only allow NaN as a sentinel value
 	if math.IsNaN(v) && !s.finished {
 		s.err = fmt.Errorf("unsupported value: NaN")
@@ -106,8 +105,8 @@ func (s *FloatEncoder) Push(v float64) {
 	} else {
 		s.bw.WriteBit(bitstream.One)
 
-		leading := bits.Clz(vDelta)
-		trailing := bits.Ctz(vDelta)
+		leading := uint64(bits.LeadingZeros64(vDelta))
+		trailing := uint64(bits.TrailingZeros64(vDelta))
 
 		// Clamp number of leading zeros to avoid overflow when encoding
 		leading &= 0x1F
@@ -139,7 +138,7 @@ func (s *FloatEncoder) Push(v float64) {
 	s.val = v
 }
 
-// FloatDecoder decodes a byte slice into multipe float64 values
+// FloatDecoder decodes a byte slice into multiple float64 values.
 type FloatDecoder struct {
 	val uint64
 
@@ -184,6 +183,7 @@ func (it *FloatDecoder) SetBytes(b []byte) error {
 	return nil
 }
 
+// Next returns true if there are remaining values to read.
 func (it *FloatDecoder) Next() bool {
 	if it.err != nil || it.finished {
 		return false
@@ -269,10 +269,12 @@ func (it *FloatDecoder) Next() bool {
 	return true
 }
 
+// Values returns the current float64 value.
 func (it *FloatDecoder) Values() float64 {
 	return math.Float64frombits(it.val)
 }
 
+// Error returns the current decoding error.
 func (it *FloatDecoder) Error() error {
 	return it.err
 }
