@@ -17,7 +17,6 @@ import (
 	"regexp"
 	"sort"
 	"sync"
-	// "sync/atomic"
 
 	"github.com/influxdata/influxdb/models"
 	"github.com/influxdata/influxdb/pkg/bytesutil"
@@ -27,17 +26,17 @@ import (
 	"github.com/influxdata/influxdb/query"
 	"github.com/influxdata/influxdb/tsdb"
 	"github.com/influxdata/influxql"
-	"github.com/uber-go/zap"
+	"go.uber.org/zap"
 )
 
 // IndexName is the name of this index.
 const IndexName = "inmem"
 
 func init() {
-	tsdb.NewInmemIndex = func(name string) (interface{}, error) { return NewIndex(name), nil }
+	tsdb.NewInmemIndex = func(name string, sfile *tsdb.SeriesFile) (interface{}, error) { return NewIndex(name, sfile), nil }
 
-	tsdb.RegisterIndex(IndexName, func(id uint64, database, path string, opt tsdb.EngineOptions) tsdb.Index {
-		return NewShardIndex(id, database, path, opt)
+	tsdb.RegisterIndex(IndexName, func(id uint64, database, path string, seriesIDSet *tsdb.SeriesIDSet, sfile *tsdb.SeriesFile, opt tsdb.EngineOptions) tsdb.Index {
+		return NewShardIndex(id, database, path, seriesIDSet, sfile, opt)
 	})
 }
 
@@ -48,11 +47,12 @@ type Index struct {
 	mu sync.RWMutex
 
 	database string
+	sfile    *tsdb.SeriesFile
+	fieldset *tsdb.MeasurementFieldSet
 
 	// In-memory metadata index, built on load and updated when new series come in
-	measurements map[string]*Measurement // measurement name to object and index
-	series       map[string]*Series      // map series key to the Series object
-	lastID       uint64                  // last used series ID. They're in memory only for this shard
+	measurements map[string]*measurement // measurement name to object and index
+	series       map[string]*series      // map series key to the Series object
 
 	seriesSketch, seriesTSSketch             *hll.Plus
 	measurementsSketch, measurementsTSSketch *hll.Plus
@@ -62,11 +62,12 @@ type Index struct {
 }
 
 // NewIndex returns a new initialized Index.
-func NewIndex(database string) *Index {
+func NewIndex(database string, sfile *tsdb.SeriesFile) *Index {
 	index := &Index{
 		database:     database,
-		measurements: make(map[string]*Measurement),
-		series:       make(map[string]*Series),
+		sfile:        sfile,
+		measurements: make(map[string]*measurement),
+		series:       make(map[string]*series),
 	}
 
 	index.seriesSketch = hll.NewDefaultPlus()
@@ -81,10 +82,15 @@ func (i *Index) Type() string      { return IndexName }
 func (i *Index) Open() (err error) { return nil }
 func (i *Index) Close() error      { return nil }
 
-func (i *Index) WithLogger(zap.Logger) {}
+func (i *Index) WithLogger(*zap.Logger) {}
+
+// Database returns the name of the database the index was initialized with.
+func (i *Index) Database() string {
+	return i.database
+}
 
 // Series returns a series by key.
-func (i *Index) Series(key []byte) (*Series, error) {
+func (i *Index) Series(key []byte) (*series, error) {
 	i.mu.RLock()
 	s := i.series[string(key)]
 	i.mu.RUnlock()
@@ -98,18 +104,8 @@ func (i *Index) SeriesSketches() (estimator.Sketch, estimator.Sketch, error) {
 	return i.seriesSketch.Clone(), i.seriesTSSketch.Clone(), nil
 }
 
-// SeriesN returns the number of unique non-tombstoned series in the index.
-// Since indexes are not shared across shards, the count returned by SeriesN
-// cannot be combined with other shards' counts.
-func (i *Index) SeriesN() int64 {
-	i.mu.RLock()
-	n := int64(len(i.series))
-	i.mu.RUnlock()
-	return n
-}
-
 // Measurement returns the measurement object from the index by the name
-func (i *Index) Measurement(name []byte) (*Measurement, error) {
+func (i *Index) Measurement(name []byte) (*measurement, error) {
 	i.mu.RLock()
 	defer i.mu.RUnlock()
 	return i.measurements[string(name)], nil
@@ -130,11 +126,11 @@ func (i *Index) MeasurementsSketches() (estimator.Sketch, estimator.Sketch, erro
 }
 
 // MeasurementsByName returns a list of measurements.
-func (i *Index) MeasurementsByName(names [][]byte) ([]*Measurement, error) {
+func (i *Index) MeasurementsByName(names [][]byte) ([]*measurement, error) {
 	i.mu.RLock()
 	defer i.mu.RUnlock()
 
-	a := make([]*Measurement, 0, len(names))
+	a := make([]*measurement, 0, len(names))
 	for _, name := range names {
 		if m := i.measurements[string(name)]; m != nil {
 			a = append(a, m)
@@ -143,60 +139,119 @@ func (i *Index) MeasurementsByName(names [][]byte) ([]*Measurement, error) {
 	return a, nil
 }
 
-// CreateSeriesIfNotExists adds the series for the given measurement to the
+// MeasurementIterator returns an iterator over all measurements in the index.
+// MeasurementIterator does not support authorization.
+func (i *Index) MeasurementIterator() (tsdb.MeasurementIterator, error) {
+	names, err := i.MeasurementNamesByExpr(nil, nil)
+	if err != nil {
+		return nil, err
+	}
+	return tsdb.NewMeasurementSliceIterator(names), nil
+}
+
+// CreateSeriesListIfNotExists adds the series for the given measurement to the
 // index and sets its ID or returns the existing series object
-func (i *Index) CreateSeriesIfNotExists(shardID uint64, key, name []byte, tags models.Tags, opt *tsdb.EngineOptions, ignoreLimits bool) error {
+func (i *Index) CreateSeriesListIfNotExists(shardID uint64, seriesIDSet *tsdb.SeriesIDSet, keys, names [][]byte, tagsSlice []models.Tags, opt *tsdb.EngineOptions, ignoreLimits bool) error {
+	seriesIDs, err := i.sfile.CreateSeriesListIfNotExists(names, tagsSlice, nil)
+	if err != nil {
+		return err
+	}
+
 	i.mu.RLock()
-	// if there is a series for this id, it's already been added
-	ss := i.series[string(key)]
+	// if there is a series for this ids, it's already been added
+	seriesList := make([]*series, len(seriesIDs))
+	for j, key := range keys {
+		seriesList[j] = i.series[string(key)]
+	}
 	i.mu.RUnlock()
 
-	if ss != nil {
-		ss.AssignShard(shardID)
+	var hasNewSeries bool
+	for _, ss := range seriesList {
+		if ss == nil {
+			hasNewSeries = true
+			continue
+		}
+
+		// This series might need to be added to the local bitset, if the series
+		// was created on another shard.
+		seriesIDSet.Lock()
+		if !seriesIDSet.ContainsNoLock(ss.ID) {
+			seriesIDSet.AddNoLock(ss.ID)
+		}
+		seriesIDSet.Unlock()
+	}
+	if !hasNewSeries {
 		return nil
 	}
 
 	// get or create the measurement index
-	m := i.CreateMeasurementIndexIfNotExists(name)
+	mms := make([]*measurement, len(names))
+	for j, name := range names {
+		mms[j] = i.CreateMeasurementIndexIfNotExists(name)
+	}
 
 	i.mu.Lock()
 	defer i.mu.Unlock()
 
 	// Check for the series again under a write lock
-	ss = i.series[string(key)]
-	if ss != nil {
-		ss.AssignShard(shardID)
+	var newSeriesN int
+	for j, key := range keys {
+		if seriesList[j] != nil {
+			continue
+		}
+
+		ss := i.series[string(key)]
+		if ss == nil {
+			newSeriesN++
+			continue
+		}
+		seriesList[j] = ss
+
+		// This series might need to be added to the local bitset, if the series
+		// was created on another shard.
+		seriesIDSet.Lock()
+		if !seriesIDSet.ContainsNoLock(ss.ID) {
+			seriesIDSet.AddNoLock(ss.ID)
+		}
+		seriesIDSet.Unlock()
+	}
+	if newSeriesN == 0 {
 		return nil
 	}
 
 	// Verify that the series will not exceed limit.
 	if !ignoreLimits {
-		if max := opt.Config.MaxSeriesPerDatabase; max > 0 && len(i.series)+1 > max {
-			return errMaxSeriesPerDatabaseExceeded
+		if max := opt.Config.MaxSeriesPerDatabase; max > 0 && len(i.series)+len(keys) > max {
+			return errMaxSeriesPerDatabaseExceeded{limit: opt.Config.MaxSeriesPerDatabase}
 		}
 	}
 
-	// set the in memory ID for query processing on this shard
-	// The series key and tags are clone to prevent a memory leak
-	series := NewSeries([]byte(string(key)), tags.Clone())
-	series.ID = i.lastID + 1
-	i.lastID++
+	for j, key := range keys {
+		if seriesList[j] != nil {
+			continue
+		}
 
-	series.SetMeasurement(m)
-	i.series[string(key)] = series
+		// set the in memory ID for query processing on this shard
+		// The series key and tags are clone to prevent a memory leak
+		skey := string(key)
+		ss := newSeries(seriesIDs[j], mms[j], skey, tagsSlice[j].Clone())
+		i.series[skey] = ss
 
-	m.AddSeries(series)
-	series.AssignShard(shardID)
+		mms[j].AddSeries(ss)
 
-	// Add the series to the series sketch.
-	i.seriesSketch.Add(key)
+		// Add the series to the series sketch.
+		i.seriesSketch.Add(key)
+
+		// This series needs to be added to the bitset tracking undeleted series IDs.
+		seriesIDSet.Add(seriesIDs[j])
+	}
 
 	return nil
 }
 
 // CreateMeasurementIndexIfNotExists creates or retrieves an in memory index
 // object for the measurement
-func (i *Index) CreateMeasurementIndexIfNotExists(name []byte) *Measurement {
+func (i *Index) CreateMeasurementIndexIfNotExists(name []byte) *measurement {
 	name = escape.Unescape(name)
 
 	// See if the measurement exists using a read-lock
@@ -216,7 +271,7 @@ func (i *Index) CreateMeasurementIndexIfNotExists(name []byte) *Measurement {
 	// and acquire the write lock
 	m = i.measurements[string(name)]
 	if m == nil {
-		m = NewMeasurement(i.database, string(name))
+		m = newMeasurement(i.database, string(name))
 		i.measurements[string(name)] = m
 
 		// Add the measurement to the measurements sketch.
@@ -238,15 +293,15 @@ func (i *Index) HasTagKey(name, key []byte) (bool, error) {
 }
 
 // HasTagValue returns true if tag value exists.
-func (i *Index) HasTagValue(name, key, value []byte) bool {
+func (i *Index) HasTagValue(name, key, value []byte) (bool, error) {
 	i.mu.RLock()
 	mm := i.measurements[string(name)]
 	i.mu.RUnlock()
 
 	if mm == nil {
-		return false
+		return false, nil
 	}
-	return mm.HasTagKeyValue(key, value)
+	return mm.HasTagKeyValue(key, value), nil
 }
 
 // TagValueN returns the cardinality of a tag value.
@@ -290,19 +345,19 @@ func (i *Index) TagKeyHasAuthorizedSeries(auth query.Authorizer, name []byte, ke
 	// possible to get the set of unique series IDs for a given measurement name
 	// and tag key.
 	var authorized bool
-	mm.SeriesByTagKeyValue(key).Range(func(_ string, seriesIDs SeriesIDs) bool {
-		if auth == nil || auth == query.OpenAuthorizer {
+	mm.SeriesByTagKeyValue(key).Range(func(_ string, sIDs seriesIDs) bool {
+		if query.AuthorizerIsOpen(auth) {
 			authorized = true
 			return false
 		}
 
-		for _, id := range seriesIDs {
+		for _, id := range sIDs {
 			s := mm.SeriesByID(id)
 			if s == nil {
 				continue
 			}
 
-			if auth.AuthorizeSeriesRead(i.database, mm.name, s.Tags()) {
+			if auth.AuthorizeSeriesRead(i.database, mm.NameBytes, s.Tags) {
 				authorized = true
 				return false
 			}
@@ -332,14 +387,14 @@ func (i *Index) MeasurementTagKeyValuesByExpr(auth query.Authorizer, name []byte
 
 	// If we haven't been provided sorted keys, then we need to sort them.
 	if !keysSorted {
-		sort.Sort(sort.StringSlice(keys))
+		sort.Strings(keys)
 	}
 
 	ids, _, _ := mm.WalkWhereForSeriesIds(expr)
 	if ids.Len() == 0 && expr == nil {
 		for ki, key := range keys {
 			values := mm.TagValues(auth, key)
-			sort.Sort(sort.StringSlice(values))
+			sort.Strings(values)
 			results[ki] = values
 		}
 		return results, nil
@@ -365,13 +420,13 @@ func (i *Index) MeasurementTagKeyValuesByExpr(auth query.Authorizer, name []byte
 		if s == nil {
 			continue
 		}
-		if auth != nil && !auth.AuthorizeSeriesRead(i.database, s.Measurement().name, s.Tags()) {
+		if auth != nil && !auth.AuthorizeSeriesRead(i.database, s.Measurement.NameBytes, s.Tags) {
 			continue
 		}
 
 		// Iterate the tag keys we're interested in and collect values
 		// from this series, if they exist.
-		for _, t := range s.Tags() {
+		for _, t := range s.Tags {
 			if idx, ok := keyIdxs[string(t.Key)]; ok {
 				resultSet[idx].add(string(t.Value))
 			} else if string(t.Key) > keys[len(keys)-1] {
@@ -429,11 +484,14 @@ func (i *Index) TagsForSeries(key string) (models.Tags, error) {
 	if ss == nil {
 		return nil, nil
 	}
-	return ss.Tags(), nil
+	return ss.Tags, nil
 }
 
 // MeasurementNamesByExpr takes an expression containing only tags and returns a
 // list of matching measurement names.
+//
+// TODO(edd): Remove authorisation from these methods. There shouldn't need to
+// be any auth passed down into the index.
 func (i *Index) MeasurementNamesByExpr(auth query.Authorizer, expr influxql.Expr) ([][]byte, error) {
 	i.mu.RLock()
 	defer i.mu.RUnlock()
@@ -443,7 +501,7 @@ func (i *Index) MeasurementNamesByExpr(auth query.Authorizer, expr influxql.Expr
 		a := make([][]byte, 0, len(i.measurements))
 		for _, m := range i.measurements {
 			if m.Authorized(auth) {
-				a = append(a, m.name)
+				a = append(a, m.NameBytes)
 			}
 		}
 		bytesutil.Sort(a)
@@ -535,7 +593,7 @@ func (i *Index) measurementNamesByNameFilter(auth query.Authorizer, op influxql.
 		}
 
 		if matched && m.Authorized(auth) {
-			names = append(names, m.name)
+			names = append(names, m.NameBytes)
 		}
 	}
 	bytesutil.Sort(names)
@@ -563,24 +621,31 @@ func (i *Index) measurementNamesByTagFilters(auth query.Authorizer, filter *TagF
 
 		tagMatch = false
 		// Authorization must be explicitly granted when an authorizer is present.
-		authorized = auth == nil
+		authorized = query.AuthorizerIsOpen(auth)
 
 		// Check the tag values belonging to the tag key for equivalence to the
 		// tag value being filtered on.
-		tagVals.Range(func(tv string, seriesIDs SeriesIDs) bool {
+		tagVals.Range(func(tv string, seriesIDs seriesIDs) bool {
 			if !valEqual(tv) {
 				return true // No match. Keep checking.
 			}
 
 			tagMatch = true
-			if auth == nil {
+			if query.AuthorizerIsOpen(auth) {
 				return false // No need to continue checking series, there is a match.
 			}
 
 			// Is there a series with this matching tag value that is
 			// authorized to be read?
 			for _, sid := range seriesIDs {
-				if s := m.SeriesByID(sid); s != nil && auth.AuthorizeSeriesRead(i.database, m.name, s.Tags()) {
+				s := m.SeriesByID(sid)
+
+				// If the series is deleted then it can't be used to authorise against.
+				if s != nil && s.Deleted() {
+					continue
+				}
+
+				if s != nil && auth.AuthorizeSeriesRead(i.database, m.NameBytes, s.Tags) {
 					// The Range call can return early as a matching
 					// tag value with an authorized series has been found.
 					authorized = true
@@ -607,7 +672,7 @@ func (i *Index) measurementNamesByTagFilters(auth query.Authorizer, filter *TagF
 		//     False  |       True      |      False
 		//     False  |       False     |      True
 		if tagMatch == (filter.Op == influxql.EQ || filter.Op == influxql.EQREGEX) && authorized {
-			names = append(names, []byte(m.Name))
+			names = append(names, m.NameBytes)
 		}
 	}
 
@@ -623,7 +688,7 @@ func (i *Index) MeasurementNamesByRegex(re *regexp.Regexp) ([][]byte, error) {
 	var matches [][]byte
 	for _, m := range i.measurements {
 		if re.MatchString(m.Name) {
-			matches = append(matches, []byte(m.Name))
+			matches = append(matches, m.NameBytes)
 		}
 	}
 	return matches, nil
@@ -654,8 +719,26 @@ func (i *Index) dropMeasurement(name string) error {
 	return nil
 }
 
-// DropSeries removes the series key and its tags from the index.
-func (i *Index) DropSeries(key []byte) error {
+// DropMeasurementIfSeriesNotExist drops a measurement only if there are no more
+// series for the measurment.
+func (i *Index) DropMeasurementIfSeriesNotExist(name []byte) error {
+	i.mu.Lock()
+	defer i.mu.Unlock()
+
+	m := i.measurements[string(name)]
+	if m == nil {
+		return nil
+	}
+
+	if m.HasSeries() {
+		return nil
+	}
+
+	return i.dropMeasurement(string(name))
+}
+
+// DropSeriesGlobal removes the series key and its tags from the index.
+func (i *Index) DropSeriesGlobal(key []byte, ts int64) error {
 	if key == nil {
 		return nil
 	}
@@ -676,21 +759,20 @@ func (i *Index) DropSeries(key []byte) error {
 	delete(i.series, k)
 
 	// Remove the measurement's reference.
-	series.Measurement().DropSeries(series)
-
+	series.Measurement.DropSeries(series)
 	// Mark the series as deleted.
 	series.Delete()
 
 	// If the measurement no longer has any series, remove it as well.
-	if !series.Measurement().HasSeries() {
-		i.dropMeasurement(series.Measurement().Name)
+	if !series.Measurement.HasSeries() {
+		i.dropMeasurement(series.Measurement.Name)
 	}
 
 	return nil
 }
 
 // TagSets returns a list of tag sets.
-func (i *Index) TagSets(shardID uint64, name []byte, opt query.IteratorOptions) ([]*query.TagSet, error) {
+func (i *Index) TagSets(shardSeriesIDs *tsdb.SeriesIDSet, name []byte, opt query.IteratorOptions) ([]*query.TagSet, error) {
 	i.mu.RLock()
 	defer i.mu.RUnlock()
 
@@ -699,7 +781,7 @@ func (i *Index) TagSets(shardID uint64, name []byte, opt query.IteratorOptions)
 		return nil, nil
 	}
 
-	tagSets, err := mm.TagSets(shardID, opt)
+	tagSets, err := mm.TagSets(shardSeriesIDs, opt)
 	if err != nil {
 		return nil, err
 	}
@@ -715,10 +797,22 @@ func (i *Index) SeriesKeys() []string {
 	}
 	i.mu.RUnlock()
 	return s
+
 }
 
 // SetFieldSet sets a shared field set from the engine.
-func (i *Index) SetFieldSet(*tsdb.MeasurementFieldSet) {}
+func (i *Index) SetFieldSet(fieldset *tsdb.MeasurementFieldSet) {
+	i.mu.Lock()
+	defer i.mu.Unlock()
+	i.fieldset = fieldset
+}
+
+// FieldSet returns the assigned fieldset.
+func (i *Index) FieldSet() *tsdb.MeasurementFieldSet {
+	i.mu.RLock()
+	defer i.mu.RUnlock()
+	return i.fieldset
+}
 
 // SetFieldName adds a field name to a measurement.
 func (i *Index) SetFieldName(measurement []byte, name string) {
@@ -729,7 +823,7 @@ func (i *Index) SetFieldName(measurement []byte, name string) {
 // ForEachMeasurementName iterates over each measurement name.
 func (i *Index) ForEachMeasurementName(fn func(name []byte) error) error {
 	i.mu.RLock()
-	mms := make(Measurements, 0, len(i.measurements))
+	mms := make(measurements, 0, len(i.measurements))
 	for _, m := range i.measurements {
 		mms = append(mms, m)
 	}
@@ -737,13 +831,112 @@ func (i *Index) ForEachMeasurementName(fn func(name []byte) error) error {
 	i.mu.RUnlock()
 
 	for _, m := range mms {
-		if err := fn([]byte(m.Name)); err != nil {
+		if err := fn(m.NameBytes); err != nil {
 			return err
 		}
 	}
 	return nil
 }
 
+func (i *Index) MeasurementSeriesIDIterator(name []byte) (tsdb.SeriesIDIterator, error) {
+	return i.MeasurementSeriesKeysByExprIterator(name, nil)
+}
+
+func (i *Index) TagKeySeriesIDIterator(name, key []byte) (tsdb.SeriesIDIterator, error) {
+	i.mu.RLock()
+	defer i.mu.RUnlock()
+
+	m := i.measurements[string(name)]
+	if m == nil {
+		return nil, nil
+	}
+	return tsdb.NewSeriesIDSliceIterator([]uint64(m.SeriesIDsByTagKey(key))), nil
+}
+
+func (i *Index) TagValueSeriesIDIterator(name, key, value []byte) (tsdb.SeriesIDIterator, error) {
+	i.mu.RLock()
+	defer i.mu.RUnlock()
+
+	m := i.measurements[string(name)]
+	if m == nil {
+		return nil, nil
+	}
+	return tsdb.NewSeriesIDSliceIterator([]uint64(m.SeriesIDsByTagValue(key, value))), nil
+}
+
+func (i *Index) TagKeyIterator(name []byte) (tsdb.TagKeyIterator, error) {
+	i.mu.RLock()
+	defer i.mu.RUnlock()
+
+	m := i.measurements[string(name)]
+	if m == nil {
+		return nil, nil
+	}
+	keys := m.TagKeys()
+	sort.Strings(keys)
+
+	a := make([][]byte, len(keys))
+	for i := range a {
+		a[i] = []byte(keys[i])
+	}
+	return tsdb.NewTagKeySliceIterator(a), nil
+}
+
+// TagValueIterator provides an iterator over all the tag values belonging to
+// series with the provided measurement name and tag key.
+//
+// TagValueIterator does not currently support authorization.
+func (i *Index) TagValueIterator(name, key []byte) (tsdb.TagValueIterator, error) {
+	i.mu.RLock()
+	defer i.mu.RUnlock()
+
+	m := i.measurements[string(name)]
+	if m == nil {
+		return nil, nil
+	}
+	values := m.TagValues(nil, string(key))
+	sort.Strings(values)
+
+	a := make([][]byte, len(values))
+	for i := range a {
+		a[i] = []byte(values[i])
+	}
+	return tsdb.NewTagValueSliceIterator(a), nil
+}
+
+func (i *Index) MeasurementSeriesKeysByExprIterator(name []byte, condition influxql.Expr) (tsdb.SeriesIDIterator, error) {
+	i.mu.RLock()
+	defer i.mu.RUnlock()
+
+	m := i.measurements[string(name)]
+	if m == nil {
+		return nil, nil
+	}
+
+	// Return all series if no condition specified.
+	if condition == nil {
+		return tsdb.NewSeriesIDSliceIterator([]uint64(m.SeriesIDs())), nil
+	}
+
+	// Get series IDs that match the WHERE clause.
+	ids, filters, err := m.WalkWhereForSeriesIds(condition)
+	if err != nil {
+		return nil, err
+	}
+
+	// Delete boolean literal true filter expressions.
+	// These are returned for `WHERE tagKey = 'tagVal'` type expressions and are okay.
+	filters.DeleteBoolLiteralTrues()
+
+	// Check for unsupported field filters.
+	// Any remaining filters means there were fields (e.g., `WHERE value = 1.2`).
+	if filters.Len() > 0 {
+		return nil, errors.New("fields not supported in WHERE clause during deletion")
+	}
+
+	return tsdb.NewSeriesIDSliceIterator([]uint64(ids)), nil
+}
+
 func (i *Index) MeasurementSeriesKeysByExpr(name []byte, condition influxql.Expr) ([][]byte, error) {
 	i.mu.RLock()
 	defer i.mu.RUnlock()
@@ -777,57 +970,27 @@ func (i *Index) MeasurementSeriesKeysByExpr(name []byte, condition influxql.Expr
 	return m.SeriesKeysByID(ids), nil
 }
 
-// SeriesPointIterator returns an influxql iterator over all series.
-func (i *Index) SeriesPointIterator(opt query.IteratorOptions) (query.Iterator, error) {
+// SeriesIDIterator returns an influxql iterator over matching series ids.
+func (i *Index) SeriesIDIterator(opt query.IteratorOptions) (tsdb.SeriesIDIterator, error) {
 	i.mu.RLock()
 	defer i.mu.RUnlock()
 
 	// Read and sort all measurements.
-	mms := make(Measurements, 0, len(i.measurements))
+	mms := make(measurements, 0, len(i.measurements))
 	for _, mm := range i.measurements {
 		mms = append(mms, mm)
 	}
 	sort.Sort(mms)
 
-	return &seriesPointIterator{
+	return &seriesIDIterator{
 		database: i.database,
 		mms:      mms,
-		point: query.FloatPoint{
-			Aux: make([]interface{}, len(opt.Aux)),
-		},
-		opt: opt,
+		opt:      opt,
 	}, nil
 }
 
-// SnapshotTo is a no-op since this is an in-memory index.
-func (i *Index) SnapshotTo(path string) error { return nil }
-
-// AssignShard update the index to indicate that series k exists in the given shardID.
-func (i *Index) AssignShard(k string, shardID uint64) {
-	ss, _ := i.Series([]byte(k))
-	if ss != nil {
-		ss.AssignShard(shardID)
-	}
-}
-
-// UnassignShard updates the index to indicate that series k does not exist in
-// the given shardID.
-func (i *Index) UnassignShard(k string, shardID uint64) error {
-	ss, _ := i.Series([]byte(k))
-	if ss != nil {
-		if ss.Assigned(shardID) {
-			// Remove the shard from any series
-			ss.UnassignShard(shardID)
-
-			// If this series no longer has shards assigned, remove the series
-			if ss.ShardN() == 0 {
-				// Remove the series key from the index.
-				return i.DropSeries([]byte(k))
-			}
-		}
-	}
-	return nil
-}
+// DiskSizeBytes always returns zero bytes, since this is an in-memory index.
+func (i *Index) DiskSizeBytes() int64 { return 0 }
 
 // Rebuild recreates the measurement indexes to allow deleted series to be removed
 // and garbage collected.
@@ -845,26 +1008,18 @@ func (i *Index) Rebuild() {
 			return nil
 		}
 
-		nm := m.Rebuild()
 		i.mu.Lock()
+		nm := m.Rebuild()
+
 		i.measurements[string(name)] = nm
 		i.mu.Unlock()
 		return nil
 	})
 }
 
-// RemoveShard removes all references to shardID from any series or measurements
-// in the index.  If the shard was the only owner of data for the series, the series
-// is removed from the index.
-func (i *Index) RemoveShard(shardID uint64) {
-	for _, k := range i.SeriesKeys() {
-		i.UnassignShard(k, shardID)
-	}
-}
-
 // assignExistingSeries assigns the existing series to shardID and returns the series, names and tags that
 // do not exists yet.
-func (i *Index) assignExistingSeries(shardID uint64, keys, names [][]byte, tagsSlice []models.Tags) ([][]byte, [][]byte, []models.Tags) {
+func (i *Index) assignExistingSeries(shardID uint64, seriesIDSet *tsdb.SeriesIDSet, keys, names [][]byte, tagsSlice []models.Tags) ([][]byte, [][]byte, []models.Tags) {
 	i.mu.RLock()
 	var n int
 	for j, key := range keys {
@@ -874,7 +1029,13 @@ func (i *Index) assignExistingSeries(shardID uint64, keys, names [][]byte, tagsS
 			tagsSlice[n] = tagsSlice[j]
 			n++
 		} else {
-			ss.AssignShard(shardID)
+			// Add the existing series to this shard's bitset, since this may
+			// be the first time the series is added to this shard.
+			seriesIDSet.Lock()
+			if !seriesIDSet.ContainsNoLock(ss.ID) {
+				seriesIDSet.AddNoLock(ss.ID)
+			}
+			seriesIDSet.Unlock()
 		}
 	}
 	i.mu.RUnlock()
@@ -888,22 +1049,45 @@ var _ tsdb.Index = &ShardIndex{}
 // in-memory index. This is required because per-shard in-memory indexes will
 // grow the heap size too large.
 type ShardIndex struct {
-	*Index
+	id uint64 // shard id
+
+	*Index // Shared reference to global database-wide index.
+
+	// Bitset storing all undeleted series IDs associated with this shard.
+	seriesIDSet *tsdb.SeriesIDSet
 
-	id  uint64 // shard id
 	opt tsdb.EngineOptions
 }
 
+// DropSeries removes the provided series id from the local bitset that tracks
+// series in this shard only.
+func (idx *ShardIndex) DropSeries(seriesID uint64, _ []byte, _ bool) error {
+	// Remove from shard-local bitset if it exists.
+	idx.seriesIDSet.Lock()
+	if idx.seriesIDSet.ContainsNoLock(seriesID) {
+		idx.seriesIDSet.RemoveNoLock(seriesID)
+	}
+	idx.seriesIDSet.Unlock()
+	return nil
+}
+
+// DropMeasurementIfSeriesNotExist drops a measurement only if there are no more
+// series for the measurment.
+func (idx *ShardIndex) DropMeasurementIfSeriesNotExist(name []byte) error {
+	return idx.Index.DropMeasurementIfSeriesNotExist(name)
+}
+
 // CreateSeriesListIfNotExists creates a list of series if they doesn't exist in bulk.
 func (idx *ShardIndex) CreateSeriesListIfNotExists(keys, names [][]byte, tagsSlice []models.Tags) error {
-	keys, names, tagsSlice = idx.assignExistingSeries(idx.id, keys, names, tagsSlice)
+	keys, names, tagsSlice = idx.assignExistingSeries(idx.id, idx.seriesIDSet, keys, names, tagsSlice)
 	if len(keys) == 0 {
 		return nil
 	}
 
-	var reason string
-	var dropped int
-	var droppedKeys map[string]struct{}
+	var (
+		reason      string
+		droppedKeys [][]byte
+	)
 
 	// Ensure that no tags go over the maximum cardinality.
 	if maxValuesPerTag := idx.opt.Config.MaxValuesPerTag; maxValuesPerTag > 0 {
@@ -914,7 +1098,7 @@ func (idx *ShardIndex) CreateSeriesListIfNotExists(keys, names [][]byte, tagsSli
 			tags := tagsSlice[i]
 			for _, tag := range tags {
 				// Skip if the tag value already exists.
-				if idx.HasTagValue(name, tag.Key, tag.Value) {
+				if ok, _ := idx.HasTagValue(name, tag.Key, tag.Value); ok {
 					continue
 				}
 
@@ -924,19 +1108,19 @@ func (idx *ShardIndex) CreateSeriesListIfNotExists(keys, names [][]byte, tagsSli
 					continue
 				}
 
-				dropped++
-				reason = fmt.Sprintf("max-values-per-tag limit exceeded (%d/%d): measurement=%q tag=%q value=%q",
-					n, maxValuesPerTag, name, string(tag.Key), string(tag.Value))
-
-				if droppedKeys == nil {
-					droppedKeys = make(map[string]struct{})
+				if reason == "" {
+					reason = fmt.Sprintf("max-values-per-tag limit exceeded (%d/%d): measurement=%q tag=%q value=%q",
+						n, maxValuesPerTag, name, string(tag.Key), string(tag.Value))
 				}
-				droppedKeys[string(keys[i])] = struct{}{}
+
+				droppedKeys = append(droppedKeys, keys[i])
 				continue outer
 			}
 
 			// Increment success count if all checks complete.
-			keys[n], names[n], tagsSlice[n] = keys[i], names[i], tagsSlice[i]
+			if n != i {
+				keys[n], names[n], tagsSlice[n] = keys[i], names[i], tagsSlice[i]
+			}
 			n++
 		}
 
@@ -944,23 +1128,15 @@ func (idx *ShardIndex) CreateSeriesListIfNotExists(keys, names [][]byte, tagsSli
 		keys, names, tagsSlice = keys[:n], names[:n], tagsSlice[:n]
 	}
 
-	// Write
-	for i := range keys {
-		if err := idx.CreateSeriesIfNotExists(keys[i], names[i], tagsSlice[i]); err == errMaxSeriesPerDatabaseExceeded {
-			dropped++
-			reason = fmt.Sprintf("max-series-per-database limit exceeded: (%d)", idx.opt.Config.MaxSeriesPerDatabase)
-			if droppedKeys == nil {
-				droppedKeys = make(map[string]struct{})
-			}
-			droppedKeys[string(keys[i])] = struct{}{}
-			continue
-		} else if err != nil {
-			return err
-		}
+	if err := idx.Index.CreateSeriesListIfNotExists(idx.id, idx.seriesIDSet, keys, names, tagsSlice, &idx.opt, idx.opt.Config.MaxSeriesPerDatabase == 0); err != nil {
+		reason = err.Error()
+		droppedKeys = append(droppedKeys, keys...)
 	}
 
 	// Report partial writes back to shard.
-	if dropped > 0 {
+	if len(droppedKeys) > 0 {
+		dropped := len(droppedKeys) // number dropped before deduping
+		bytesutil.SortDedup(droppedKeys)
 		return &tsdb.PartialWriteError{
 			Reason:      reason,
 			Dropped:     dropped,
@@ -971,59 +1147,72 @@ func (idx *ShardIndex) CreateSeriesListIfNotExists(keys, names [][]byte, tagsSli
 	return nil
 }
 
-// InitializeSeries is called during start-up.
-// This works the same as CreateSeriesIfNotExists except it ignore limit errors.
-func (i *ShardIndex) InitializeSeries(key, name []byte, tags models.Tags) error {
-	return i.Index.CreateSeriesIfNotExists(i.id, key, name, tags, &i.opt, true)
+// SeriesN returns the number of unique non-tombstoned series local to this shard.
+func (idx *ShardIndex) SeriesN() int64 {
+	idx.mu.RLock()
+	defer idx.mu.RUnlock()
+	return int64(idx.seriesIDSet.Cardinality())
 }
 
-func (i *ShardIndex) CreateSeriesIfNotExists(key, name []byte, tags models.Tags) error {
-	return i.Index.CreateSeriesIfNotExists(i.id, key, name, tags, &i.opt, false)
+// InitializeSeries is called during start-up.
+// This works the same as CreateSeriesListIfNotExists except it ignore limit errors.
+func (idx *ShardIndex) InitializeSeries(keys, names [][]byte, tags []models.Tags) error {
+	return idx.Index.CreateSeriesListIfNotExists(idx.id, idx.seriesIDSet, keys, names, tags, &idx.opt, true)
+}
+
+// CreateSeriesIfNotExists creates the provided series on the index if it is not
+// already present.
+func (idx *ShardIndex) CreateSeriesIfNotExists(key, name []byte, tags models.Tags) error {
+	return idx.Index.CreateSeriesListIfNotExists(idx.id, idx.seriesIDSet, [][]byte{key}, [][]byte{name}, []models.Tags{tags}, &idx.opt, false)
 }
 
 // TagSets returns a list of tag sets based on series filtering.
-func (i *ShardIndex) TagSets(name []byte, opt query.IteratorOptions) ([]*query.TagSet, error) {
-	return i.Index.TagSets(i.id, name, opt)
+func (idx *ShardIndex) TagSets(name []byte, opt query.IteratorOptions) ([]*query.TagSet, error) {
+	return idx.Index.TagSets(idx.seriesIDSet, name, opt)
+}
+
+// SeriesIDSet returns the bitset associated with the series ids.
+func (idx *ShardIndex) SeriesIDSet() *tsdb.SeriesIDSet {
+	return idx.seriesIDSet
 }
 
 // NewShardIndex returns a new index for a shard.
-func NewShardIndex(id uint64, database, path string, opt tsdb.EngineOptions) tsdb.Index {
+func NewShardIndex(id uint64, database, path string, seriesIDSet *tsdb.SeriesIDSet, sfile *tsdb.SeriesFile, opt tsdb.EngineOptions) tsdb.Index {
 	return &ShardIndex{
-		Index: opt.InmemIndex.(*Index),
-		id:    id,
-		opt:   opt,
+		Index:       opt.InmemIndex.(*Index),
+		id:          id,
+		seriesIDSet: seriesIDSet,
+		opt:         opt,
 	}
 }
 
-// seriesPointIterator emits series as influxql points.
-type seriesPointIterator struct {
+// seriesIDIterator emits series ids.
+type seriesIDIterator struct {
 	database string
-	mms      Measurements
+	mms      measurements
 	keys     struct {
-		buf []*Series
+		buf []*series
 		i   int
 	}
-
-	point query.FloatPoint // reusable point
-	opt   query.IteratorOptions
+	opt query.IteratorOptions
 }
 
 // Stats returns stats about the points processed.
-func (itr *seriesPointIterator) Stats() query.IteratorStats { return query.IteratorStats{} }
+func (itr *seriesIDIterator) Stats() query.IteratorStats { return query.IteratorStats{} }
 
 // Close closes the iterator.
-func (itr *seriesPointIterator) Close() error { return nil }
+func (itr *seriesIDIterator) Close() error { return nil }
 
 // Next emits the next point in the iterator.
-func (itr *seriesPointIterator) Next() (*query.FloatPoint, error) {
+func (itr *seriesIDIterator) Next() (tsdb.SeriesIDElem, error) {
 	for {
 		// Load next measurement's keys if there are no more remaining.
 		if itr.keys.i >= len(itr.keys.buf) {
 			if err := itr.nextKeys(); err != nil {
-				return nil, err
+				return tsdb.SeriesIDElem{}, err
 			}
 			if len(itr.keys.buf) == 0 {
-				return nil, nil
+				return tsdb.SeriesIDElem{}, nil
 			}
 		}
 
@@ -1031,23 +1220,16 @@ func (itr *seriesPointIterator) Next() (*query.FloatPoint, error) {
 		series := itr.keys.buf[itr.keys.i]
 		itr.keys.i++
 
-		if !itr.opt.Authorizer.AuthorizeSeriesRead(itr.database, series.measurement.name, series.tags) {
+		if !itr.opt.Authorizer.AuthorizeSeriesRead(itr.database, series.Measurement.NameBytes, series.Tags) {
 			continue
 		}
 
-		// Write auxiliary fields.
-		for i, f := range itr.opt.Aux {
-			switch f.Val {
-			case "key":
-				itr.point.Aux[i] = series.Key
-			}
-		}
-		return &itr.point, nil
+		return tsdb.SeriesIDElem{SeriesID: series.ID}, nil
 	}
 }
 
 // nextKeys reads all keys for the next measurement.
-func (itr *seriesPointIterator) nextKeys() error {
+func (itr *seriesIDIterator) nextKeys() error {
 	for {
 		// Ensure previous keys are cleared out.
 		itr.keys.i, itr.keys.buf = 0, itr.keys.buf[:0]
@@ -1079,4 +1261,10 @@ func (itr *seriesPointIterator) nextKeys() error {
 
 // errMaxSeriesPerDatabaseExceeded is a marker error returned during series creation
 // to indicate that a new series would exceed the limits of the database.
-var errMaxSeriesPerDatabaseExceeded = errors.New("max series per database exceeded")
+type errMaxSeriesPerDatabaseExceeded struct {
+	limit int
+}
+
+func (e errMaxSeriesPerDatabaseExceeded) Error() string {
+	return fmt.Sprintf("max-series-per-database limit exceeded: (%d)", e.limit)
+}
